
package org.gjt.bugrat.db;

import java.io.Serializable;
import java.util.Date;
import java.util.Vector;

import org.gjt.bugrat.dbi.*;


public
class		BugRatAction
extends		JartDBObject
implements	Cloneable, BugRatConstants
	{
	private int			id;
	private String		type;
	private int			actor;
	private Date		timestamp;
	private String		target;
	private int			targetId;
	private int			argone;
	private int			argtwo;
	private String		comment;


	public static BugRatAction
	getAction( int id )
		throws DBIException
		{
		return DBIManager.getInstance().getActionDBI().getAction( id );
		}

	public static BugRatAction
	getNewAction( String type )
		throws DBIException
		{
		return DBIManager.getInstance().getActionDBI().getNewAction( type );
		}

	public static Vector
	getActions( String target, int id )
		throws DBIException
		{
		return DBIManager.getInstance().getActionDBI().getActions( target, id );
		}

	public
	BugRatAction( int id, String type )
		{
		this.id = id;
		this.type = type;
		this.actor = 0;
		this.timestamp = null;
		this.target = TARGET_NULL;
		this.targetId = 0;
		this.argone = 0;
		this.argtwo = 0;
		this.comment = null;
		}

	public int
	getId()
		{
		return this.id;
		}

	public String
	getType()
		{
		return this.type;
		}

	public int
	getActorId()
		{
		return this.actor;
		}

	public void
	setActorId( int actor )
		{
		this.actor = actor;
		}

	public Date
	getActionTime()
		{
		return this.timestamp;
		}

	public void
	setActionTime( Date timestamp )
		{
		this.timestamp = timestamp;
		}

	public String
	getTarget()
		{
		return this.target;
		}

	public String
	getTargetName()
		{
		if ( this.target.equals( TARGET_BUG ) )
			return "Bug";
		else if ( this.target.equals( TARGET_REPORT ) )
			return "Report";
		else if ( this.target.equals( TARGET_PERSON ) )
			return "Person";
		else if ( this.target.equals( TARGET_CATEGORY ) )
			return "Category";
		else if ( this.target.equals( TARGET_COMMENT ) )
			return "Comment";
		else if ( this.target.equals( TARGET_PROPERTY ) )
			return "Property";
		else if ( this.target.equals( TARGET_DESCRIPTION ) )
			return "Description";
		else
			return "NullTarget";
		}

	public void
	setTarget( String target )
		{
		this.target = target;
		}

	public int
	getTargetId()
		{
		return this.targetId;
		}

	public void
	setTargetId( int targetId )
		{
		this.targetId = targetId;
		}

	public int
	getArgOne()
		{
		return this.argone;
		}

	public void
	setArgOne( int argument )
		{
		this.argone = argument;
		}

	public int
	getArgTwo()
		{
		return this.argtwo;
		}

	public void
	setArgTwo( int argument )
		{
		this.argtwo = argument;
		}

	public String
	getComment()
		{
		return this.comment;
		}

	public void
	setComment( String comment )
		{
		this.comment = comment;
		}

	public void
	commit()
		throws DBIException
		{
		this.getDBI().commit( this );
		this.setModified( false );
		this.setInDatabase( true );
		}

	public Object
	clone()
		{
		BugRatAction act = (BugRatAction) super.clone();		

		act.id = this.id;
		act.type = new String( this.type );

		act.actor = this.actor;
		act.timestamp = new Date( this.timestamp.getTime() );

		act.target = new String( this.target );
		act.targetId = this.targetId;

		act.argone = this.argone;
		act.argtwo = this.argtwo;
		act.comment = new String( this.comment );

		return act;
		}

	/**
	 * Returns the currently set ReportDBI, which is the class that
	 * performs our database needs in a generic fashion.
	 *
	 * @return The ReportDBI subclass that is currently set.
	 * @see ReportDBI
	 */

	private ActionDBI
	getDBI()
		throws DBIException
		{
		return DBIManager.getInstance().getActionDBI();
		}

	public String
	toString()
		{
		return "[BugRatAction: id=" + this.id + ",type=" + this.type + "]";
		}

	public String
	dumpString()
		{
		return "["
			+ super.toString()
			+ " id=" + this.id
			+ " type=" + this.type
			+ " actor=" + this.actor
			+ " timestamp=" + this.timestamp
			+ " target=" + this.target
			+ " targetId=" + this.targetId
			+ " argone=" + this.argone
			+ " argtwo=" + this.argtwo
			+ " comment=" + this.comment
			+ "]";
		}

	}

