
package org.gjt.bugrat.db;

import java.io.Serializable;
import java.util.Vector;
import java.util.Hashtable;

import org.gjt.bugrat.dbi.*;


public class
Category extends JartDBObject
	implements Cloneable
	{
	private String		pkg;
	private String		cat;
	private String		sub;

	private int			responsible;

	private String		name;
	private String		desc;


	public static Category
	getCategory( String pkg, String cat, String sub )
		throws DBIException
		{
		return DBIManager.getInstance().getCategoryDBI().getCategory( pkg, cat, sub);
		}

	public static Vector
	getAllCategories()
		throws DBIException
		{
		return DBIManager.getInstance().getCategoryDBI().getAllCategories();
		}

	public
	Category( String pkg, String cat, String sub )
		{
		this.pkg = pkg;
		this.cat = cat;
		this.sub = sub;
		}

	public String
	getProject()
		{
		return this.pkg;
		}

	/**
	 * Determines if this category is a Project category.
	 *
	 * NOTE Oracle is a very brain damaged DB, which can not distinguish
	 *      between a null string and an empty string. Ergo, we changed
	 *      the definition of an "empty" value to be ".". However, we
	 *      wanted to support existing databases, so we will check for
	 *      the new "." and the old "", as well as null.
	 */
	public boolean
	isProject()
		{
		return
			(  ( this.cat == null || this.cat.length() == 0 || this.cat.equals( "." ) )
			&& ( this.sub == null || this.sub.length() == 0 || this.sub.equals( "." ) ) );
		}

	public String
	getCategory()
		{
		return this.cat;
		}

	/**
	 * Determines if this category is a category.
	 *
	 * See note under isProject().
	 */
	public boolean
	isCategory()
		{
		return
			(  ( this.cat != null && this.cat.length() > 0 && !this.cat.equals( "." ) )
			&& ( this.sub == null || this.sub.length() == 0 || this.sub.equals( "." ) ) );
		}

	public String
	getSubCategory()
		{
		return this.sub;
		}

	/**
	 * Determines if this category is a subcategory.
	 *
	 * See note under isProject().
	 */
	public boolean
	isSubCategory()
		{
		return ( this.sub != null && this.sub.length() > 0 && !this.sub.equals( "." ) );
		}

	public int
	getResponsible()
		{
		return this.responsible;
		}

	public void
	setResponsible( int resp )
		{
		this.responsible = resp;
		}

	public String
	getName()
		{
		return this.name;
		}

	public void
	setName( String name )
		{
		this.name = name;
		}

	public String
	getDescription()
		{
		return this.desc;
		}

	public void
	setDescription( String desc )
		{
		this.desc = desc;
		}

	public void
	commit()
		throws DBIException
		{
		DBIManager.getInstance().getCategoryDBI().commit( this );
		this.setModified( false );
		this.setInDatabase( true );
		}

	public Object
	clone()
		{
		Category cat = (Category) super.clone();		

		cat.pkg = this.pkg;
		cat.cat = this.cat;
		cat.sub = this.sub;
		cat.name = this.name;
		cat.desc = this.desc;

		return desc;
		}

	public String
	dumpString()
		{
		return "[" + super.toString()
			+ " pkg=" + this.pkg
			+ " cat=" + this.cat
			+ " sub=" + this.sub + "]";
		}

	public String
	toString()
		{
		return this.getName();
		}

	}

