
package org.gjt.bugrat.db;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Dimension;
import java.util.*;

import org.gjt.bugrat.db.*;
import org.gjt.bugrat.dbi.*;
import com.ice.util.StringUtilities;


public class
DBConfig
	{
	private static DBConfig		instance = null;
	private static Object		synchro = new Object();

	private Hashtable		persons = null;
	private Hashtable		properties = null;
	private Hashtable		tagTable = null;

	private Vector			projects = null;
	private Vector			categories = null;
	private Vector			subcats = null;

	private Vector			priorities = null;
	private Vector			confidences = null;
	private Vector			severities = null;
	private Vector			classes = null;
	private Vector			states = null;

	private Vector			sortedPersons = null;

	private Vector			configChangeListeners = null;


	public
	DBConfig()
		{
		this.persons = new Hashtable();
		this.tagTable = new Hashtable();

		this.projects = new Vector();
		this.categories = new Vector();
		this.subcats = new Vector();

		this.priorities = new Vector();
		this.confidences = new Vector();
		this.severities = new Vector();
		this.classes = new Vector();
		this.states = new Vector();

		this.sortedPersons = null;

		this.configChangeListeners = new Vector();
		}

	//
	// CLASS METHODS
	//

	public static DBConfig
	getInstance()
		throws DBIException
		{
		synchronized ( DBConfig.synchro )
			{
			if ( DBConfig.instance == null )
				{
				DBConfig.instance = new DBConfig();
				DBConfig.instance.retrieveConfiguration();
				}
			}

		return DBConfig.instance;
		}

	//
	// INSTANCE METHODS
	//

	public void
	addConfigChangeListener( DBConfigChangeListener listener )
		{
		this.configChangeListeners.addElement( listener );
		}

	public void
	removeConfigChangeListener( DBConfigChangeListener listener )
		{
		this.configChangeListeners.removeElement( listener );
		}

	public Hashtable
	getPersons()
		{
		return this.persons;
		}

	public Vector
	getSortedPersons()
		{
		if ( this.sortedPersons == null )
			{
			this.sortedPersons = new Vector( this.persons.size() );
			Enumeration enum = this.persons.elements();
			for ( ; enum.hasMoreElements() ; )
				{
				boolean inserted = false;
				Person p = (Person) enum.nextElement();
				String n1 = p.getName().toLowerCase();

				for ( int i = 0, sz = this.sortedPersons.size()
						; i < sz ; ++i )
					{
					Person vp = (Person) this.sortedPersons.elementAt(i);
					String n2 = vp.getName().toLowerCase();
					if ( n1.compareTo( n2 ) < 0 )
						{
						inserted = true;
						this.sortedPersons.insertElementAt( p, i );
						break;
						}
					}

				if ( ! inserted )
					{
					this.sortedPersons.addElement( p );
					}
				}
			}

		return this.sortedPersons;
		}

	public Vector
	getProjects()
		{
		return this.projects;
		}

	public Vector
	getCategories()
		{
		return this.categories;
		}

	public Vector
	getSubCats()
		{
		return this.subcats;
		}

	public Vector
	getPriorities()
		{
		return this.priorities;
		}

	public Vector
	getConfidences()
		{
		return this.confidences;
		}

	public Vector
	getSeverities()
		{
		return this.severities;
		}

	public Vector
	getClasses()
		{
		return this.classes;
		}

	public Vector
	getStates()
		{
		return this.states;
		}

	public String
	getAdminEmailAddress()
		{
		return this.getProperty
			( "global.adminAddress", "BugRatAdmin" );
		}

	public Enumeration
	getPropertyEnumerator()
		{
		return this.properties.elements();
		}

	public String
	getProperty( String key )
		{
		return this.getProperty( key, null );
		}

	public String
	getProperty( String key, String defValue )
		{
		String result = null;
		
		BRATProperty prop = (BRATProperty)
			this.properties.get( key );

		if ( prop == null )
			{
			result = defValue;
			}
		else
			{
			result = prop.getValue();
			}

		return result;
		}

	public boolean
	getBoolean( String key, boolean defValue )
		{
		boolean result = false;
		
		BRATProperty prop = (BRATProperty)
			this.properties.get( key );

		if ( prop == null )
			{
			result = defValue;
			}
		else
			{
			String val = prop.getValue();
			if ( val.equalsIgnoreCase( "t" )
					|| val.equalsIgnoreCase( "true" )
					|| val.equalsIgnoreCase( "y" )
					|| val.equalsIgnoreCase( "yes" )
					|| val.equalsIgnoreCase( "1" ) )
				{
				result = true;
				}
			}

		return result;
		}

	public int
	getInteger( String key, int defValue )
		{
		int result = defValue;
		
		BRATProperty prop = (BRATProperty)
			this.properties.get( key );

		if ( prop == null )
			{
			result = defValue;
			}
		else
			{
			String val = prop.getValue();
			try { result = Integer.parseInt( prop.getValue() ); }
			catch ( NumberFormatException ex )
				{ result = defValue; }
			}

		return result;
		}

	public String
	getProjectName( String projectId )
		{
		for ( int i = 0, sz = this.projects.size() ; i < sz ; ++i )
			{
			Category c = (Category) this.projects.elementAt(i);
			if ( projectId.equals( c.getProject() ) )
				return c.getName();
			}

		return null;
		}

	public String
	getCategoryName( String proId, String catId )
		{
		for ( int i = 0, sz = this.categories.size() ; i < sz ; ++i )
			{
			Category c = (Category) this.categories.elementAt(i);
			if (	   c.getProject().equals( proId )
					&& c.getCategory().equals( catId ) )
				{
				return c.getName();
				}
			}

		return null;
		}

	public String
	getSubCategoryName( String proId, String catId, String subId )
		{
		for ( int i = 0, sz = this.subcats.size() ; i < sz ; ++i )
			{
			Category c = (Category) this.subcats.elementAt(i);
			if (	   c.getProject().equals( proId )
					&& c.getCategory().equals( catId )
					&& c.getSubCategory().equals( subId ) )
				{
				return c.getName();
				}
			}

		return null;
		}

	public String
	getClassName( String classId )
		{
		for ( int i = 0, sz = this.classes.size() ; i < sz ; ++i )
			{
			LevelParam lp = (LevelParam) this.classes.elementAt(i);
			if ( classId.equals( lp.getId() ) )
				{
				return lp.getName();
				}
			}

		return "unknown class '" + classId + "'";
		}

	public String
	getPriorityName( String priorId )
		{
		for ( int i = 0, sz = this.priorities.size() ; i < sz ; ++i )
			{
			LevelParam lp = (LevelParam) this.priorities.elementAt(i);
			if ( priorId.equals( lp.getId() ) )
				{
				return lp.getName();
				}
			}

		return "unknown priority '" + priorId + "'";
		}

	public String
	getSeverityName( String severId )
		{
		for ( int i = 0, sz = this.severities.size() ; i < sz ; ++i )
			{
			LevelParam lp = (LevelParam) this.severities.elementAt(i);
			if ( severId.equals( lp.getId() ) )
				{
				return lp.getName();
				}
			}

		return "unknown severity '" + severId + "'";
		}

	public String
	getStateName( String stateId )
		{
		for ( int i = 0, sz = this.states.size() ; i < sz ; ++i )
			{
			LevelParam lp = (LevelParam) this.states.elementAt(i);
			if ( stateId.equals( lp.getId() ) )
				{
				return lp.getName();
				}
			}

		return "unknown state '" + stateId + "'";
		}

	public String
	getConfidenceName( String confId )
		{
		for ( int i = 0, sz = this.confidences.size() ; i < sz ; ++i )
			{
			LevelParam lp = (LevelParam) this.confidences.elementAt(i);
			if ( confId.equals( lp.getId() ) )
				{
				return lp.getName();
				}
			}

		return "unknown confidence '" + confId + "'";
		}

	/**
	 * Returns the default state for a new report.
	 *
	 * @return The id of the default state.
	 */
	public String
	getDefaultReportState()
		{
		return this.getProperty( "global.defaultReportState", "R" );
		}

	/**
	 * Returns the default confidence level for a new report.
	 *
	 * @return The id of the default confidence level.
	 */
	public String
	getDefaultConfidence()
		{
		return this.getProperty( "global.defaultReportConfidence", "P" );
		}

	/**
	 * Returns the default responsible person. Used for incoming
	 * reports, and other cases where no responsible party can
	 * be found.
	 *
	 * @return The id of the default confidence level.
	 */
	public int
	getDefaultResponsible()
		{
		return this.getInteger( "global.defReportResponsible", 0 );
		}

	/**
	 * This method is called to get the default "responsible" person
	 * for a new bug report. This person would then be responsible for
	 * reassigning the report to the proper person. This works hierarchically.
	 * That is, it first looks for some responsible for the specific subcategory.
	 * If no person is configured as responsible for the subcategory, then the
	 * category will be consulted, and if that has no person assigned, then
	 * the project will be consulted. If none of these category levels has
	 * an assigned responsible person, then the default setting in the DB
	 * configuration will be consulted for a global value. If all of these
	 * fails to produce a responsible party, sigh, we leave it unassigned,
	 * which is id zero.
	 *
	 * @param pkgStr The name of the project of the new report.
	 * @param catStr The name of the category of the new report.
	 * @param subStr The name of the subcategory of the new report.
	 * @return The id of the person responsible.
	 */

	public int
	getIncomingResponsible( String pkgStr, String catStr, String subStr )
		{
		int result = 0;

		Category rCat = null;
		
		try {
			rCat = Category.getCategory( pkgStr, catStr, subStr );
			if ( rCat == null || rCat.getResponsible() == 0 )
				{
				rCat = Category.getCategory( pkgStr, catStr, "" );
				if ( rCat == null || rCat.getResponsible() == 0 )
					{
					rCat = Category.getCategory( pkgStr, "", "" );
					}
				}

			if ( rCat != null && rCat.getResponsible() != 0 )
				{
				result = rCat.getResponsible();
				}
			else
				{
				result = this.getDefaultResponsible();
				}
			}
		catch ( DBIException ex )
			{
			// REVIEW Should we be throwing this exception?
			result = 0;
			}

		return result;
		}

	public Vector
	getLevels( String table )
		{
		if ( table.equals( "severity" ) )
			{
			return this.severities;
			}
		else if ( table.equals( "priority" ) )
			{
			return this.priorities;
			}
		else if ( table.equals( "state" ) )
			{
			return this.states;
			}
		else if ( table.equals( "class" ) )
			{
			return this.classes;
			}
		else if ( table.equals( "confidence" ) )
			{
			return this.confidences;
			}

		(new Throwable( "UNKNOWN table '" +table + "'" )).
			printStackTrace( System.err );

		return new Vector();
		}

	public Vector
	getProjectCategories( String project )
		{
		Vector result = new Vector();
		for ( int i = 0, sz = this.categories.size()
				; i < sz ; ++i )
			{
			Category c = (Category)
				this.categories.elementAt(i);

			if ( c.getProject().equals( project ) )
				result.addElement( c );
			}

		return result;
		}

	public Vector
	getCategorySubCats( String project, String category )
		{
		Vector result = new Vector();
		for ( int i = 0, sz = this.subcats.size()
				; i < sz ; ++i )
			{
			Category c = (Category)
				this.subcats.elementAt(i);

			if ( c.getProject().equals( project )
					&& c.getCategory().equals( category ) )
				result.addElement( c );
			}

		return result;
		}

	public boolean
	isHTMLTag( String tag )
		{
		tag = tag.toLowerCase();
		return ( this.tagTable.get( tag ) != null );
		}

	public boolean
	isTextHTML( String text )
		throws DBIException
		{
		DBConfig config = DBConfig.getInstance();

		for ( int off = 0 ; ; )
			{
			int beg = text.indexOf( "<", off );
			if ( beg == -1 )
				break;

			int end = text.indexOf( ">", beg );
			if ( end == -1 )
				break;

			off = end + 1;
			String tag = text.substring( beg + 1, end );

			if ( tag.startsWith( "/" ) )
				tag = tag.substring( 1 );

			int idx = tag.indexOf( " " );
			if ( idx > 0 )
				tag = tag.substring( 0, idx );

			if ( config.isHTMLTag( tag ) )
				return true;
			}

		return false;
		}

	public void
	retrievePersons()
		throws DBIException
		{
		this.persons = Person.getAllPersons();
		this.sortedPersons = null;
		}

	public void
	retrieveCategories()
		throws DBIException
		{
		Vector allcats = Category.getAllCategories();
		
		this.projects = new Vector();
		this.categories = new Vector();
		this.subcats = new Vector();

		Enumeration enum = allcats.elements();
		for ( ; enum.hasMoreElements() ; )
			{
			Category c = (Category) enum.nextElement();
			if ( c.isProject() )
				{
				this.projects.addElement( c );
				}
			else if ( c.isCategory() )
				{
				this.categories.addElement( c );
				}
			else
				{
				this.subcats.addElement( c );
				}
			}
		}

	public void
	retrieveConfiguration()
		throws DBIException
		{
		this.retrievePersons();

		this.retrieveCategories();

		this.priorities =
			LevelParam.getAllLevelParams
				( LevelParam.PRIORITY_TABLE );

		this.severities =
			LevelParam.getAllLevelParams
				( LevelParam.SEVERITY_TABLE );

		this.states =
			LevelParam.getAllLevelParams
				( LevelParam.STATE_TABLE );

		this.confidences =
			LevelParam.getAllLevelParams
				( LevelParam.CONFIDENCE_TABLE );

		this.classes =
			LevelParam.getAllLevelParams
				( LevelParam.CLASS_TABLE );

		this.properties =
			BRATProperty.getAllProperties();

		String tagStr =
			this.getProperty( "servletCommon.htmlTags" );

		if ( tagStr != null )
			{
			StringTokenizer toker =
				new StringTokenizer( tagStr, "," );

			for ( ; toker.hasMoreTokens() ; )
				{
				try {
					String tag = toker.nextToken();
					this.tagTable.put( tag, tag );
					}
				catch ( NoSuchElementException ex )
					{ break; }
				}
			}

		synchronized ( this.configChangeListeners )
			{
			for ( int i = 0, sz = this.configChangeListeners.size()
					; i < sz ; ++i )
				{
				DBConfigChangeListener listener =
					(DBConfigChangeListener)
						this.configChangeListeners.elementAt( i );

				listener.dbConfigHasChanged();
				}
			}
		}

	public String
	formatPerson( int id )
		{
		String result = "Unidentified Person, ID = " + id;

		try {
			Person p = Person.getPerson( id );
			if ( p != null )
				{
				result =
					p.getName()
					+ " (&nbsp;<a href=\"mailto:"
					+ p.getEmail()
					+ "\">"
					+ p.getEmail()
					+ "</a>&nbsp;)";
				}
			}
		catch ( DBIException ex )
			{
			ex.printStackTrace();
			result += " (Exception: " + ex.getMessage() + ")";
			}

		return result;
		}

	}

