
package org.gjt.bugrat.db;

import java.io.Serializable;
import java.util.Hashtable;

import org.gjt.bugrat.dbi.*;


public class
EnvDescription extends JartDBObject
	implements Cloneable
	{
	private int				descId;

	private String			release;
	private String			jvm;
	private String			os;
	private String			osrel;
	private String			platform;
	private String			envdesc;


	public static EnvDescription
	getEnvDescription( int id )
		throws DBIException
		{
		EnvDescription envDesc =
			DBIManager.getInstance().getDescriptionDBI().getEnvDescription( id );
		return envDesc;
		}

	public static EnvDescription
	getNewEnvDescription()
		throws DBIException
		{
		return DBIManager.getInstance().getDescriptionDBI().getNewEnvDescription();
		}

	public
	EnvDescription( int id )
		{
		this.descId = id;
		}

	public int
	getId()
		{
		return this.descId;
		}

	public String
	getRelease()
		{
		return this.release;
		}

	public void
	setRelease( String release )
		{
		this.release = release;
		}

	public String
	getJVM()
		{
		return this.jvm;
		}

	public void
	setJVM( String jvm )
		{
		this.jvm = jvm;
		}

	public String
	getOS()
		{
		return this.os;
		}

	public void
	setOS( String os )
		{
		this.os = os;
		}

	public String
	getOSRelease()
		{
		return this.osrel;
		}

	public void
	setOSRelease( String osrel )
		{
		this.osrel = osrel;
		}

	public String
	getPlatform()
		{
		return this.platform;
		}

	public void
	setPlatform( String platform )
		{
		this.platform = platform;
		}

	public String
	getDescription()
		{
		return this.envdesc;
		}

	public void
	setDescription( String desc )
		{
		this.envdesc = desc;
		}

	public void
	commit()
		throws DBIException
		{
		this.getDBI().commit( this );
		}

	public Object
	clone()
		{
		EnvDescription desc =
			(EnvDescription) super.clone();		

		desc.descId = this.descId;
		desc.release = this.release;
		desc.os = this.os;
		desc.osrel = this.osrel;
		desc.platform = this.platform;
		desc.envdesc = this.envdesc;

		return desc;
		}

	public boolean
	equals( EnvDescription that )
		{
		if ( that == null )
			return false;

		if ( this.descId != that.descId )
			return false;

		if ( ! this.nullEqualStrings( this.release, that.release ) )
			return false;
		if ( ! this.nullEqualStrings( this.jvm, that.jvm ) )
			return false;
		if ( ! this.nullEqualStrings( this.os, that.os ) )
			return false;
		if ( ! this.nullEqualStrings( this.osrel, that.osrel ) )
			return false;
		if ( ! this.nullEqualStrings( this.platform, that.platform ) )
			return false;
		if ( ! this.nullEqualStrings( this.envdesc, that.envdesc ) )
			return false;

		return true;
		}

	public StringBuffer
	getDeltaDescription(
			EnvDescription that, String prefix,
			StringBuffer buf, DBConfig cfg )
		{
		if ( that == null )
			{
			buf.append( prefix + "New environment description:\n" );
			buf.append( prefix + "       Release: " + this.release + "\n" );
			buf.append( prefix + "           JVM: " + this.jvm + "\n" );
			buf.append( prefix + "            OS: " + this.os + "\n" );
			buf.append( prefix + "    OS Release: " + this.osrel + "\n" );
			buf.append( prefix + "      Platform: " + this.platform + "\n" );
			buf.append( prefix + "   Description: " + this.envdesc + "\n" );
			return buf;
			}

		if ( ! this.nullEqualStrings( this.release, that.release ) )
			{
			buf.append( prefix );
			buf.append( "Release changed from '" );
			buf.append( that.release );
			buf.append( "' to '" );
			buf.append( this.release );
			buf.append( "'.\n" );
			}

		if ( ! this.nullEqualStrings( this.jvm, that.jvm ) )
			{
			buf.append( prefix );
			buf.append( "JVM changed from '" );
			buf.append( that.jvm );
			buf.append( "' to '" );
			buf.append( this.jvm );
			buf.append( "'.\n" );
			}

		if ( ! this.nullEqualStrings( this.os, that.os ) )
			{
			buf.append( prefix );
			buf.append( "OS changed from '" );
			buf.append( that.os );
			buf.append( "' to '" );
			buf.append( this.os );
			buf.append( "'.\n" );
			}

		if ( ! this.nullEqualStrings( this.osrel, that.osrel ) )
			{
			buf.append( prefix );
			buf.append( "OS Release changed from '" );
			buf.append( that.osrel );
			buf.append( "' to '" );
			buf.append( this.osrel );
			buf.append( "'.\n" );
			}

		if ( ! this.nullEqualStrings( this.platform, that.platform ) )
			{
			buf.append( prefix );
			buf.append( "Platform changed from '" );
			buf.append( that.platform );
			buf.append( "' to '" );
			buf.append( this.platform );
			buf.append( "'.\n" );
			}

		if ( ! this.nullEqualStrings( this.envdesc, that.envdesc ) )
			{
			buf.append( prefix );
			buf.append( "Description changed from:\n" );
			buf.append( prefix );
			buf.append( " > " );
			buf.append( that.envdesc );
			buf.append( "\n" );
			buf.append( prefix );
			buf.append( "To:\n" );
			buf.append( prefix );
			buf.append( " > " );
			buf.append( this.envdesc );
			buf.append( "\n" );
			}

		return buf;
		}

	private DescriptionDBI
	getDBI()
		{
		DescriptionDBI dbi = null;

		try {
			dbi = DBIManager.getInstance().getDescriptionDBI();
			}
		catch ( DBIException ex )
			{
			ex.printStackTrace( System.err );
			}

		return dbi;
		}

	public String
	toString()
		{
		return "[" + super.toString()
			+ " descId=" + this.descId + "]";
		}

	}

