
package org.gjt.bugrat.db;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Enumeration;

import org.gjt.bugrat.dbi.*;


/**
 * This Class is subclassed by all objects in the Jart database.
 * The Class provides functionality common to all objects, such
 * as indicating if the object is in the database, and if the
 * object has been modified.
 *
 */


public
class		JartDBObject
extends		Object
implements	Serializable, Cloneable
	{
	private boolean			isInDB;
	private boolean			isMod;


	public
	JartDBObject()
		{
		this.isInDB = false;
		this.isMod = false;
		}

	/**
	 * Determine if this object is in the database, or new.
	 * This method does <em>not</em> cause db access.
	 *
	 * @return True if the object is currently in the database.
	 */

	public boolean
	isInDatabase()
		{
		return this.isInDB;
		}

	public void
	setInDatabase( boolean isInDB )
		{
		this.isInDB = isInDB;
		}

	/**
	 * The 'Modified' methods here are a convenience.
	 * It is expected that each subclass, except for the
	 * simplest objects, will override these methods to
	 * allow for tracking "parts" that are modified to
	 * prevent updating "the whole world" just to change
	 * an "address".
	 *
	 * The isModified() method determines if this object
	 * has been modified and need to be committed.
	 *
	 * @return True if object has been modified, else false.
	 */

	public boolean
	isModified()
		{
		return this.isMod;
		}

	/**
	 * Sets the modified flag for this object.
	 *
	 * @param isMod The new 'modified' setting.
	 */

	public void
	setModified( boolean isMod )
		{
		this.isMod = isMod;
		}

	/**
	 * Compares two strings, either of which may be null.
	 */

	public boolean
	nullEqualStrings( String s1, String s2 )
		{
		if ( s1 == null && s2 != null )
			return false;

		if ( s1 != null && s2 == null )
			return false;

		if ( s1 != null && s2 != null )
			return s1.equals( s2 );

		return true;
		}

	public Object
	clone()
		{
		JartDBObject jo = null;

		try {
			jo = (JartDBObject) super.clone();
			jo.setModified( this.isMod );
			jo.setInDatabase( this.isInDB );
			}
		catch ( CloneNotSupportedException ex )
			{
			ex.printStackTrace( System.err );
			}
			
		return jo;
		}

	public String
	toString()
		{
		return super.toString() + "[isMod=" + this.isMod
			+ ",isInDB=" + this.isInDB + "]";
		}
	}

