
package org.gjt.bugrat.dbi;

import java.sql.*;
import java.util.Date;
import java.util.Vector;
import org.gjt.bugrat.db.*;


public
class		ActionJDBC
extends		BugRatJDBC
implements	ActionDBI
	{
	private static final boolean	DEBUG = false;

	public
	ActionJDBC( JDBCHandler handler )
		{
		super( handler );
		}

	public BugRatAction
	getAction( int id )
		throws DBIException
		{
		BugRatAction result = null;
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   rataction.acttype," +
				"   rataction.actor," +
				"   rataction.acttime," +
				"   rataction.target," +
				"   rataction.targetid," +
				"   rataction.argone," +
				"   rataction.argtwo," +
				"   rataction.actcmt " +
				"FROM rataction " +
				"WHERE rataction.id = '" + id + "'";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				String type = rs.getString( col++ );

				result = new BugRatAction( id, type );
				result.setModified( false );
				result.setInDatabase( true );

				result.setActorId( rs.getInt( col++ ) );
				result.setActionTime
					( this.localizeDate( rs.getTimestamp( col++ ) ) );
				result.setTarget( rs.getString( col++ ) );
				result.setTargetId( rs.getInt( col++ ) );
				result.setArgOne( rs.getInt( col++ ) );
				result.setArgTwo( rs.getInt( col++ ) );
				result.setComment( rs.getString( col++ ) );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			throw new DBIException
				( ex, "get action, statement '" + queryStr + "'" );
			}

		return result;
		}

	public Vector
	getActions( String target, int targetId )
		throws DBIException
		{
		Vector result = new Vector();
		String queryStr =
			"SELECT rataction.id FROM rataction " +
			" WHERE rataction.target = '" + target + "' " +
			"   AND rataction.targetid = '" + targetId + "'";

		try {
			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			for ( ; rs.next() ; )
				{
				int col = 1;
				int actId = rs.getInt( col++ );

				if ( actId > 0 )
					{
					BugRatAction act = BugRatAction.getAction( actId );
					result.addElement( act );
					}
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public BugRatAction
	getNewAction( String type )
		throws DBIException
		{
		String queryStr = null;
		BugRatAction result = null;

		try {
			int actId = this.getNextId( "action" );

			StringBuffer buf = new StringBuffer( 400 );

			buf.append( "INSERT INTO rataction " );
			buf.append( " ( " );
			buf.append( "  rataction.id," );
			buf.append( "  rataction.acttype," );
			buf.append( "  rataction.actor," );
			buf.append( "  rataction.acttime," );
			buf.append( "  rataction.target," );
			buf.append( "  rataction.targetid," );
			buf.append( "  rataction.argone," );
			buf.append( "  rataction.argtwo," );
			buf.append( "  rataction.actcmt " );
			buf.append( " ) " );
			buf.append( "VALUES " );
			buf.append( " ( ?, ?, ?, ?, ?, ?, ?, ?, ? )" );

			PreparedStatement pS =
				this.handler.createPreparedStatement( buf.toString() );

			int pidx = 1;
			pS.setInt( pidx++, actId );
			pS.setString( pidx++, BugRatAction.ACTION_NOOP );
			pS.setInt( pidx++, 0 );
			pS.setTimestamp( pidx++, this.normalizeDate( new Date() ) );
			pS.setString( pidx++, BugRatAction.TARGET_NULL );
			pS.setInt( pidx++, 0 );
			pS.setInt( pidx++, 0 );
			pS.setInt( pidx++, 0 );
			pS.setNull( pidx++, java.sql.Types.VARCHAR );

			int rows = pS.executeUpdate();

			pS.close();

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not insert new action" );
				}

			result = new BugRatAction( actId, type );
			result.setInDatabase( true );
			}
		catch ( SQLException ex )
			{
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public void
	commit( BugRatAction act )
		throws DBIException
		{
		try {
			StringBuffer buf = new StringBuffer();

			if ( act.isInDatabase() )
				{
				buf.append( "UPDATE rataction " );
				buf.append( "SET" );
				buf.append( "  rataction.acttype = ?," );
				buf.append( "  rataction.actor = ?," );
				buf.append( "  rataction.acttime = ?," );
				buf.append( "  rataction.target = ?," );
				buf.append( "  rataction.targetid = ?," );
				buf.append( "  rataction.argone = ?," );
				buf.append( "  rataction.argtwo = ?," );
				buf.append( "  rataction.actcmt = ? " );
				buf.append( "WHERE" );
				buf.append( "  rataction.id = '" + act.getId() + "' " );
				}
			else
				{
				buf.append( "INSERT INTO rataction" );
				buf.append( " ( id, acttype, actor, acttime, target," );
				buf.append( "   targetid, argone, argtwo, actcmt ) " );
				buf.append( "VALUES" );
				buf.append( " ( ?, ?, ?, ?, ?, ?, ?, ?, ? )" );
				}

			if ( DEBUG )
				System.err.println
					( "ActionJDBC.commit: isInDatabase = '"
						+ act.isInDatabase() + "'  Cmd = '"
						+ buf.toString() + "'" );

            PreparedStatement pS =
				this.handler.createPreparedStatement
					( buf.toString() );
			
			int pidx = 1;
			if ( ! act.isInDatabase() )
				{
				pS.setInt( pidx++, act.getId() );
				}
			
			pS.setString( pidx++, act.getType() );
			pS.setInt( pidx++, act.getActorId() );

			java.sql.Timestamp sqlTS =
				new java.sql.Timestamp
					( act.getActionTime().getTime() );

			pS.setTimestamp( pidx++, sqlTS );

			pS.setString( pidx++, act.getTarget() );
			pS.setInt( pidx++, act.getTargetId() );
			pS.setInt( pidx++, act.getArgOne() );
			pS.setInt( pidx++, act.getArgTwo() );

			this.setNullString
				( pS, pidx++, act.getComment(), java.sql.Types.CHAR );

			int rows = pS.executeUpdate();
			
			pS.close();

			if ( DEBUG )
				System.err.println
					( "ActionJDBC.commit: rows = '" + rows + "'" );
			}
		catch ( SQLException ex )
			{
			throw new DBIException
				( ex, "committing action '" + act + "'" );
			}
		}

	}


