
package org.gjt.bugrat.dbi;

import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.text.FieldPosition;
import java.text.ParsePosition;
import java.util.Date;
import java.util.Vector;
import java.util.Hashtable;

import org.gjt.bugrat.db.*;


public class
BugJDBC extends BugRatJDBC
	implements BugDBI
	{
	private static final boolean	DEBUG = false;

	public
	BugJDBC( JDBCHandler handler )
		{
		super( handler );
		}

	public Bug
	getNewBug()
		throws DBIException
		{
		int rows;
		Bug result = null;
		String queryStr = null;

		try {
			//
			// CREATE DESCRIPTION RECORD
			//
			int bugDescId = this.getNextId( "description" );

			queryStr =
				"INSERT INTO description " +
				"  ( id, synopsis ) " +
				"VALUES " +
				"  ( " + bugDescId + ", '' ) ";

			Statement stmt = this.handler.createStatement();
			rows = stmt.executeUpdate( queryStr );
			stmt.close();
			if ( DEBUG )
				System.err.println( "ADDING BUG DESC rows=" + rows );

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not insert description" );
				}

			Description desc = new Description( bugDescId );

			//
			// CREATE ENVDESC RECORD
			//
			int envDescId = this.getNextId( "envdesc" );

			queryStr =
				"INSERT INTO envdesc " +
				"  ( id ) " +
				"VALUES " +
				"  ( " + envDescId + " ) ";

			stmt = this.handler.createStatement();
			rows = stmt.executeUpdate( queryStr );
			stmt.close();
			if ( DEBUG )
				System.err.println( "ADDING ENV DESC rows=" + rows );

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not insert environment description" );
				}

			//
			// CREATE BUG RECORD
			//
			int bugId = this.getNextId( "bug" );

			queryStr =
				"INSERT INTO bug " +
				"  ( id, bugdesc, envdesc ) " +
				"VALUES ( " +
				   bugId +
				", " +
				   bugDescId +
				", " +
				   envDescId +
				" ) ";

			stmt = this.handler.createStatement();
			rows = stmt.executeUpdate( queryStr );
			stmt.close();
			if ( DEBUG )
				System.err.println( "CREATING NEW BUG rows=" + rows );

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not create new bug" );
				}

			if ( DEBUG )
				System.err.println
					( "GET BUG id=" + bugId + " descId=" + bugDescId
						+ " envDescId=" + envDescId );

			return this.getBug( bugId );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}
		}

	public Bug
	getBug( int bugId )
		throws DBIException
		{
		Bug result = null;
		String queryStr = null;

		int descId = 0;
		int envDescId = 0;
		int reproDescId = 0;
		int aroundDescId = 0;

		try {
			queryStr = 
				"SELECT" +
				"   bug.source," +
				"   bug.srcid," +
				"   bug.responsible," +
				"   bug.opened," +
				"   bug.closed," +
				"   bug.confidence," +
				"   bug.severity," +
				"   bug.priority," +
				"   bug.state," +
				"   bug.class," +
				"   bug.project," +
				"   bug.category," +
				"   bug.subcat," +
				"   bug.bugdesc," +
				"   bug.envdesc," +
				"   bug.repro," +
				"   bug.around " +
				"FROM bug " +
				"WHERE bug.id = '" + bugId + "'";

			Statement stmt =
				this.handler.createStatement();

			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				result = new Bug( bugId );

				result.setModified( false );
				result.setInDatabase( true );

				result.setSource( rs.getString( col++ ) );
				result.setSourceId( rs.getInt( col++ ) );
				result.setResponsible( rs.getInt( col++ ) );

				result.setOpenDate
					( this.localizeDate( rs.getTimestamp( col++ ) ) );

				result.setCloseDate
					( this.localizeDate( rs.getTimestamp( col++ ) ) );

				result.setConfidence( rs.getString( col++ ) );
				result.setSeverity( rs.getString( col++ ) );
				result.setPriority( rs.getString( col++ ) );
				result.setState( rs.getString( col++ ) );
				result.setBugClass( rs.getString( col++ ) );
				result.setProject( rs.getString( col++ ) );
				result.setCategory( rs.getString( col++ ) );
				result.setSubCategory( rs.getString( col++ ) );

				descId = rs.getInt( col++ );
				envDescId = rs.getInt( col++ );
				reproDescId = rs.getInt( col++ );
				aroundDescId = rs.getInt( col++ );
				}

			rs.close();
			stmt.close();

			if ( descId > 0 )
				{
				result.setDescription
					( Description.getDescription( descId ) );
				}

			if ( envDescId > 0 )
				{
				result.setEnvDescription
					( EnvDescription.getEnvDescription( envDescId ) );
				}

			if ( reproDescId > 0 )
				{
				result.setReproDescription
					( Description.getDescription( reproDescId ) );
				}

			if ( aroundDescId > 0 )
				{
				result.setAroundDescription
					( Description.getDescription( aroundDescId ) );
				}
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public Vector
	getAllBugs()
		throws DBIException
		{
		return this.matchBugs( "" );
		}

	public Vector
	matchBugs( String selectStr )
		throws DBIException
		{
		Vector result = new Vector();
		String queryStr = null;

		try {
			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( selectStr );

			for ( ; rs.next() ; )
				{
				int col = 1;
				int bugId = rs.getInt( col++ );
				Bug bug = this.getBug( bugId );
				result.addElement( bug );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + selectStr + "'" );
			}

		return result;
		}

	public void
	commit( Bug bug )
		throws DBIException
		{
		int bugId = bug.getId();

		try {
			StringBuffer buf = new StringBuffer();

			int bugDescId = 0;
			int envDescId = 0;

			Description bugDesc = bug.getDescription();
			if ( bugDesc != null )
				{
				bugDesc.commit();
				bugDescId = bugDesc.getId();
				}

			EnvDescription envDesc = bug.getEnvDescription();
			if ( envDesc != null )
				{
				envDesc.commit();
				envDescId = envDesc.getId();
				}

			if ( bug.getReproDescription() != null )
				bug.getReproDescription().commit();

			if ( bug.getAroundDescription() != null )
				bug.getAroundDescription().commit();

			if ( bug.isInDatabase() )
				{
				buf.append( "UPDATE bug " );
				buf.append( "SET " );
				buf.append( "  bug.source = ?," );
				buf.append( "  bug.srcid = ?," );
				buf.append( "  bug.responsible = ?," );
				buf.append( "  bug.opened = ?," );
				buf.append( "  bug.closed = ?," );
				buf.append( "  bug.project = ?," );
				buf.append( "  bug.category = ?," );
				buf.append( "  bug.subcat = ?," );
				buf.append( "  bug.class = ?," );
				buf.append( "  bug.state = ?," );
				buf.append( "  bug.severity = ?," );
				buf.append( "  bug.priority = ?," );
				buf.append( "  bug.confidence = ?," );
				buf.append( "  bug.bugdesc = ?," );
				buf.append( "  bug.envdesc = ?," );
				buf.append( "  bug.repro = ?," );
				buf.append( "  bug.around = ? " );
				buf.append( "WHERE bug.id = '" + bugId + "' " );
				}
			else
				{
				buf.append( "INSERT INTO bug " );
				buf.append( "( " );
				buf.append( "  bug.id," );
				buf.append( "  bug.source," );
				buf.append( "  bug.srcid," );
				buf.append( "  bug.responsible," );
				buf.append( "  bug.opened," );
				buf.append( "  bug.closed," );
				buf.append( "  bug.project," );
				buf.append( "  bug.category," );
				buf.append( "  bug.subcat," );
				buf.append( "  bug.class," );
				buf.append( "  bug.state," );
				buf.append( "  bug.severity," );
				buf.append( "  bug.priority," );
				buf.append( "  bug.confidence," );
				buf.append( "  bug.bugdesc," );
				buf.append( "  bug.envdesc," );
				buf.append( "  bug.repro," );
				buf.append( "  bug.around " );
				buf.append( ") " );
				buf.append( "VALUES " );
				buf.append( "( ?, ?, ?, ?, ?, ?, ?, ?, ?," );
				buf.append( "  ?, ?, ?, ?, ?, ?, ?, ?, ? )" );
				}

			if ( DEBUG )
				System.err.println
					( "BugJDBC.commit: isInDatabase = '"
						+ bug.isInDatabase() + "'  Cmd = '"
						+ buf.toString() + "'" );

			PreparedStatement pS =
				this.handler.createPreparedStatement( buf.toString() );

			int pidx = 1;
			if ( ! bug.isInDatabase() )
				{
				pS.setInt( pidx++, bugId );
				}

			pS.setString( pidx++, bug.getSource() );
			pS.setInt( pidx++, bug.getSourceId() );
			pS.setInt( pidx++, bug.getResponsible() );

			pS.setTimestamp
				( pidx++, this.normalizeDate(  bug.getOpenDate() ) );

			if ( bug.getCloseDate() == null )
				pS.setNull( pidx++, java.sql.Types.TIMESTAMP );
			else
				pS.setTimestamp
					( pidx++, this.normalizeDate(  bug.getCloseDate() ) );

			pS.setString( pidx++, bug.getProject() );
			pS.setString( pidx++, bug.getCategory() );
			pS.setString( pidx++, bug.getSubCategory() );
			pS.setString( pidx++, bug.getBugClass() );
			pS.setString( pidx++, bug.getState() );
			pS.setString( pidx++, bug.getSeverity() );
			pS.setString( pidx++, bug.getPriority() );
			pS.setString( pidx++, bug.getConfidence() );

			if ( bug.getDescription() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, bug.getDescription().getId() );

			if ( bug.getEnvDescription() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, bug.getEnvDescription().getId() );

			if ( bug.getReproDescription() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, bug.getReproDescription().getId() );

			if ( bug.getAroundDescription() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, bug.getAroundDescription().getId() );

			int rows = pS.executeUpdate();
			
			pS.close();

			if ( DEBUG )
				System.err.println
					( "BugJDBC.commit: rows = '" + rows + "'" );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "committing bug '" + bugId + "'" );
			}
		}

	}


