
package org.gjt.bugrat.dbi;

import java.sql.*;
import java.util.Enumeration;
import java.util.Hashtable;

import org.gjt.bugrat.db.*;


public
class		DBAdminJDBC
extends		BugRatJDBC
implements	DBAdminDBI
	{
	private static final boolean	DEBUG = false;

	public
	DBAdminJDBC( JDBCHandler handler )
		{
		super( handler );
		}

	public Integer
	getSequenceId( String name )
		throws DBIException
		{
		String queryStr = null;
		Integer result = new Integer(1);

		try {
			queryStr = 
				"SELECT" +
				"   nextseq " +
				"FROM sequence " +
				"WHERE id = '" + name + "'";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				result = new Integer( rs.getInt( col++ ) );
				}

			rs.close();
			stmt.close();

			return result;
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}
		}

	public Hashtable
	getAllSequenceIds()
		throws DBIException
		{
		String queryStr = null;
		Hashtable result = new Hashtable();

		try {
			queryStr = 
				"SELECT" +
				"   sequence.name, " +
				"   sequence.nextseq " +
				"FROM sequence";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			for ( ; rs.next() ; )
				{
				int col = 1;
				String name = rs.getString( col++ );
				int nextseq = rs.getInt( col++ );
				result.put( name, new Integer( nextseq ) );
				}

			rs.close();
			stmt.close();

			return result;
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}
		}

	public void
	setSequenceId( String name, Integer nextSeq  )
		throws DBIException
		{
		try {
			boolean inDB = false;
			String queryStr = 
				"SELECT" +
				"   nextseq " +
				"FROM sequence " +
				"WHERE id = '" + name + "'";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );
			if ( rs.next() )
				{
				inDB = true;
				}
			rs.close();
			stmt.close();

			StringBuffer buf = new StringBuffer();

			if ( inDB )
				{
				buf.append( "UPDATE sequence " );
				buf.append( "SET" );
				buf.append( "  nextseq = ? " );
				buf.append( "WHERE" );
				buf.append( "  name = '" );
				buf.append( name );
				buf.append( "' " );
				}
			else
				{
				buf.append( "INSERT INTO sequence " );
				buf.append( " ( name, nextseq ) " );
				buf.append( "VALUES " );
				buf.append( " ( ?, ? ) " );
				}

			if ( DEBUG )
				System.err.println
					( "DBAdmin.setSequenceId: isInDatabase = '"
						+ inDB + "'  Cmd = '"
						+ buf.toString() + "'" );

			PreparedStatement pS =
				this.handler.createPreparedStatement
					( buf.toString() );

			int pidx = 1;
			if ( ! inDB )
				{
				pS.setString( pidx++, name );
				}

			pS.setInt( pidx++, nextSeq.intValue() );

			int rows = pS.executeUpdate();
			
			pS.close();

			if ( DEBUG )
				System.err.println
					( "DBAdminJDBC.setSequenceId: rows = '" + rows + "'" );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "committing new Sequence ["
					+ name + "] " );
			}
		}

	public void
	setSequenceIds( Hashtable ids )
		throws DBIException
		{
		Enumeration enum = ids.keys();
		for ( ; enum.hasMoreElements() ; )
			{
			String name = (String) enum.nextElement();
			Integer nextSeq = (Integer) ids.get( name );
			this.setSequenceId( name, nextSeq );
			}
		}

	public void
	adjustSequenceId( String idName, String tableName )
		throws DBIException
		{
		ResultSet rs = null;
		Statement stmt = null;
		PreparedStatement pS = null;

		try {
			StringBuffer qBuf = new StringBuffer(256);
			qBuf.append( "SELECT" );
			qBuf.append( "  MAX(" );
			qBuf.append( tableName );
			qBuf.append( ".id) " );
			qBuf.append( "FROM " );
			qBuf.append( tableName );

			// WARNING
			// This code is a little tricky. We close the ResultSet
			// and Statement, *only* if we intend to reuse them,
			// otherwise, we leave it to the finally block to close
			// things up for us, which allows us to throw that
			// DBIException without worrying about cleanup...
			//
			stmt = this.handler.createStatement();
			rs = stmt.executeQuery( qBuf.toString() );
			if ( rs.next() )
				{
				int nextseq = rs.getInt( 1 ) + 1;

				rs.close(); rs = null;
				stmt.close(); stmt = null;

				if ( DEBUG )
					System.err.println
						( "ADJUST SEQUENCES: name '" + idName
							+ "' for table '" + tableName
							+ "' new nextseq = " + nextseq );

				qBuf.setLength( 0 );
				qBuf.append( "SELECT" );
				qBuf.append( "  sequence.nextseq " );
				qBuf.append( "FROM " );
				qBuf.append( "  sequence " );
				qBuf.append( "WHERE " );
				qBuf.append( "   sequence.name = '" );
				qBuf.append( idName );
				qBuf.append( "'" );

				stmt = this.handler.createStatement();
				rs = stmt.executeQuery( qBuf.toString() );
				boolean hasRow = rs.next();
				if ( DEBUG && hasRow )
					{
					int currseq = rs.getInt( 1 );
					System.err.println
						( "DBAdminJDBC.adjustSequenceId: "
							+ "current nextseq=" + currseq );
					}

				rs.close(); rs = null;
				stmt.close(); stmt = null;

				qBuf.setLength( 0 );
				if ( hasRow )
					{
					qBuf.append( "UPDATE " );
					qBuf.append( "   sequence " );
					qBuf.append( "SET " );
					qBuf.append( "   sequence.nextseq = ? " );
					qBuf.append( "WHERE " );
					qBuf.append( "   sequence.name = '" );
					qBuf.append( idName );
					qBuf.append( "' " );

					pS = this.handler.createPreparedStatement( qBuf.toString() );

					pS.setInt( 1, nextseq );
					}
				else
					{
					qBuf.append( "INSERT INTO sequence " );
					qBuf.append( "  ( name, nextseq ) " );
					qBuf.append( "VALUES " );
					qBuf.append( "  ( ?, ? ) " );

					pS = this.handler.createPreparedStatement( qBuf.toString() );

					pS.setString( 1, idName );
					pS.setInt( 2, nextseq );
					}

				int rows = pS.executeUpdate();

				if ( DEBUG )
				System.err.println
					( "DBAdminJDBC.adjustSequenceId: ROWS = '" + rows + "'" );
				}
			else
				{
				if ( DEBUG )
					System.err.println
						( "DBAdminJDBC.adjustSequenceId: table '"
							+ tableName + "' appears to be empty." );
				}
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "adjusting sequenceId '" + idName
					+ "' for table '" + tableName + "'" );
			}
		finally
			{
			try {
				if ( rs != null ) rs.close();
				if ( pS != null ) pS.close();
				if ( stmt != null ) stmt.close();
				}
			catch ( SQLException ex ) {}
			}
		}

	public void
	adjustSequenceIds()
		throws DBIException
		{
		this.adjustSequenceId( "action",		"rataction" );
		this.adjustSequenceId( "bug",			"bug" );
		this.adjustSequenceId( "comment",		"ratcomment" );
		this.adjustSequenceId( "description",	"description" );
		this.adjustSequenceId( "email",			"email" );
		this.adjustSequenceId( "envdesc",		"envdesc" );
		this.adjustSequenceId( "forward",		"forward" );
		this.adjustSequenceId( "person",		"person" );
		this.adjustSequenceId( "report",		"report" );
		this.adjustSequenceId( "subscribe",		"subscribe" );
		}

	}


