
package org.gjt.bugrat.dbi;

import java.sql.*;
import java.util.Properties;

import org.gjt.bugrat.util.BugRatLogger;


public class
DBIManager extends Object
	{
	// DBI Types
	public static final int		NONE = -1;
	public static final int		JSRV = 1;
	public static final int		JDBC = 2;

	// Singleton Instance
	private static DBIManager	instance = null;


	// Logger
	private BugRatLogger		logger = null;

	// Current Manager type
	private int					currManager = NONE;

	// JDBC DBI Settings
	private JDBCHandler			handler = null;
	private String				jdbcUrl = null;
	private Properties			jdbcProps = null;
	private String				jdbcDrvrClassName = null;

	// JSRV DBI Settings
	private String				jsrvURL = null;
	private String				jsrvUser = null;
	private String				jsrvPass = null;

	// DBI List
	private BugDBI				bugDBI = null;
	private PropertyDBI			propDBI = null;
	private CategoryDBI			catDBI = null;
	private DescriptionDBI		descDBI = null;
	private EmailDBI			mailDBI = null;
	private LevelParamDBI		levelDBI = null;
	private ReportDBI			reportDBI = null;
	private PersonDBI			personDBI = null;
	private ActionDBI			actionDBI = null;
	private DBAdminDBI			dbAdminDBI = null;


	public
	DBIManager()
		{
		}

	public static DBIManager
	getInstance()
		{
		if ( DBIManager.instance == null )
			{
			DBIManager.instance = new DBIManager();
			}

		return DBIManager.instance;
		}

	public void
	resetDBIManager()
		throws DBIException
		{
		this.setCurrentDBIManager( this.currManager );
		}

	public void
	setCurrentDBIManager( int manager )
		throws DBIException
		{
		switch ( manager )
			{
			//
			// C S A P I
			//
			case DBIManager.JSRV:
				if ( this.jsrvURL == null )
					throw new DBIException
						( "Servlet Parameters not set. "
							+ "Call setJServParameters()." );

				this.loadServlet();
				break;

			//
			// J D B C
			//
			case DBIManager.JDBC:
				if ( this.jdbcUrl == null )
					throw new DBIException
						( "JDBC Parameters not set. "
							+ "Call setJDBCParameters()." );

				this.loadJDBC();
				break;
			}
		}

	public void
	setJDBCParameters(
			String url, String className,
			Properties props, BugRatLogger logger )
		{
		this.logger = logger;
		this.jdbcUrl = url;
		this.jdbcProps = props;
		this.jdbcDrvrClassName = className;
		}
/***
	public void
	setServletURL( String url )
		{
		this.jsrvURL = url;
		this.jsrvUser = null;
		this.jsrvPass = null;
		}

	public void
	setServletParameters(
			String url, String user, String pass, BugRatLogger logger )
		{
		this.logger = logger;
		this.jsrvURL = url;
		this.jsrvUser = user;
		this.jsrvPass = pass;
		}
***/

	public void
	closeDBIManager()
		{
		switch ( this.currManager )
			{
			case DBIManager.JSRV:
				// REVIEW
				break;

			case DBIManager.JDBC:
				if ( this.handler != null )
					{
					this.handler.closeConnection();
					this.handler = null;
					}
				break;
			}

		this.bugDBI = null;
		this.catDBI = null;
		this.descDBI = null;
		this.mailDBI = null;
		this.levelDBI = null;
		this.personDBI = null;
		this.reportDBI = null;
		this.propDBI = null;
		this.actionDBI = null;
		this.dbAdminDBI = null;
		}

	private void
	loadServlet()
		{
		String url = this.jsrvURL;
		String user = this.jsrvUser;
		String pass = this.jsrvPass;

		this.currManager = DBIManager.JSRV;

		this.bugDBI = new BugJServ( url, user, pass );
		this.descDBI = new DescriptionJServ( url, user, pass );
		this.mailDBI = new EmailJServ( url, user, pass );
		this.catDBI = new CategoryJServ( url, user, pass );
		this.levelDBI = new LevelParamJServ( url, user, pass );
		this.personDBI = new PersonJServ( url, user, pass );
		this.reportDBI = new ReportJServ( url, user, pass );
		this.propDBI = new PropertyJServ( url, user, pass );
		this.actionDBI = new ActionJServ( url, user, pass );
		this.dbAdminDBI = new DBAdminJServ( url, user, pass );
		}

	private void
	loadJDBC()
		throws DBIException
		{
		JDBCHandler h = null;
		this.currManager = DBIManager.JDBC;

		try {
			h = new JDBCHandler
				( this.jdbcDrvrClassName, this.jdbcUrl,
					this.jdbcProps, this.logger );

			h.openConnection();
			}
		catch ( SQLException ex )
			{
			ex.printStackTrace();
			throw new DBIException
				( ex, "could not open the JDBC connection" );
			}
		catch ( ClassNotFoundException ex )
			{
			throw new DBIException
				( ex, "CLASS NOT FOUND: could not load JDBC driver" );
			}

		this.handler = h;

		this.bugDBI = new BugJDBC( h );
		this.catDBI = new CategoryJDBC( h );
		this.descDBI = new DescriptionJDBC( h );
		this.mailDBI = new EmailJDBC( h );
		this.levelDBI = new LevelParamJDBC( h );
		this.personDBI = new PersonJDBC( h );
		this.reportDBI = new ReportJDBC( h );
		this.propDBI = new PropertyJDBC( h );
		this.actionDBI = new ActionJDBC( h );
		this.dbAdminDBI = new DBAdminJDBC( h );
		}

	public BugDBI
	getBugDBI()
		throws DBIException
		{
		if ( this.bugDBI == null )
			{
			throw new DBIException
				( "Bug DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.bugDBI;
		}

	public CategoryDBI
	getCategoryDBI()
		throws DBIException
		{
		if ( this.catDBI == null )
			{
			throw new DBIException
				( "Category DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.catDBI;
		}

	public DescriptionDBI
	getDescriptionDBI()
		throws DBIException
		{
		if ( this.descDBI == null )
			{
			throw new DBIException
				( "Description DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.descDBI;
		}

	public EmailDBI
	getEmailDBI()
		throws DBIException
		{
		if ( this.mailDBI == null )
			{
			throw new DBIException
				( "Email DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.mailDBI;
		}

	public LevelParamDBI
	getLevelParamDBI()
		throws DBIException
		{
		if ( this.levelDBI == null )
			{
			throw new DBIException
				( "LevelParam DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.levelDBI;
		}

	public ReportDBI
	getReportDBI()
		throws DBIException
		{
		if ( this.reportDBI == null )
			{
			throw new DBIException
				( "Report DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.reportDBI;
		}

	public PersonDBI
	getPersonDBI()
		throws DBIException
		{
		if ( this.personDBI == null )
			{
			throw new DBIException
				( "Person DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.personDBI;
		}

	public PropertyDBI
	getPropertyDBI()
		throws DBIException
		{
		if ( this.propDBI == null )
			{
			throw new DBIException
				( "Property DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.propDBI;
		}

	public ActionDBI
	getActionDBI()
		throws DBIException
		{
		if ( this.actionDBI == null )
			{
			throw new DBIException
				( "Property DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.actionDBI;
		}

	public DBAdminDBI
	getDBAdminDBI()
		throws DBIException
		{
		if ( this.actionDBI == null )
			{
			throw new DBIException
				( "Property DBI has not yet been set. "
					+ " You must call setCurrentDBIManager()." );
			}

		return this.dbAdminDBI;
		}

	}
