
package org.gjt.bugrat.dbi;

import java.sql.*;
import java.util.Hashtable;
import java.util.Vector;
import org.gjt.bugrat.db.*;


public
class		DescriptionJDBC
extends		BugRatJDBC
implements	DescriptionDBI
	{
	private static final boolean	DEBUG = false;

	public
	DescriptionJDBC( JDBCHandler handler )
		{
		super( handler );
		}

	public Description
	getDescription( int id )
		throws DBIException
		{
		Description result = null;
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   description.synopsis," +
				"   description.url," +
				"   description.email," +
				"   description.mimetype," +
				"   description.desctext " +
				"FROM description " +
				"WHERE description.id = '" + id + "'";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				result = new Description( id );

				result.setModified( false );
				result.setInDatabase( true );

				result.setSynopsis( rs.getString( col++ ) );
				result.setURL( this.getString( rs, col++ ) );
				result.setEmailId( rs.getInt( col++ ) );
				result.setMimeType( this.getString( rs, col++ ) );
				result.setDescription( this.getLargeString( rs, col++ ) );
				result.setEmail( null );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}


	public Description
	getNewDescription( )
		throws DBIException
		{
		Description result = null;
		String updStr = null;
		int rows;

		try {
			int descId = this.getNextId( "description" );

			updStr =
				"INSERT INTO description " +
				"  ( id, synopsis ) " +
				"VALUES " +
				"  ( " + descId + ", '' ) ";

			Statement stmt = this.handler.createStatement();
			rows = stmt.executeUpdate( updStr );
			stmt.close();

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not insert description" );
				}

			result = new Description( descId );
			result.setInDatabase( true );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + updStr + "'" );
			}

		return result;
		}

	public EnvDescription
	getEnvDescription( int id )
		throws DBIException
		{
		EnvDescription result = null;
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   envdesc.release," +
				"   envdesc.envjvm," +
				"   envdesc.envos," +
				"   envdesc.envosrel," +
				"   envdesc.envplat," +
				"   envdesc.envdesc " +
				"FROM envdesc " +
				"WHERE envdesc.id = " + id;

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				result = new EnvDescription( id );

				result.setModified( false );
				result.setInDatabase( true );

				result.setRelease( this.getString( rs, col++ ) );
				result.setJVM( this.getString( rs, col++ ) );
				result.setOS( this.getString( rs, col++ ) );
				result.setOSRelease( this.getString( rs, col++ ) );
				result.setPlatform( this.getString( rs, col++ ) );
				result.setDescription( this.getString( rs, col++ ) );
				}
			else
				{
				throw new DBIException
					( "no environment description with ID '" + id + "'" );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public EnvDescription
	getNewEnvDescription( )
		throws DBIException
		{
		EnvDescription result = null;
		String updStr = null;
		int rows;

		try {
			int descId = this.getNextId( "envdesc" );

			updStr =
				"INSERT INTO envdesc " +
				"  ( envdesc ) " +
				"VALUES " +
				"  ( " + descId + ", '' ) ";

			Statement stmt = this.handler.createStatement();
			rows = stmt.executeUpdate( updStr );
			stmt.close();

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not insert env description" );
				}

			result = new EnvDescription( descId );
			result.setInDatabase( true );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + updStr + "'" );
			}

		return result;
		}

	public Comment
	getComment( int id )
		throws DBIException
		{
		Comment result = null;
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   ratcomment.target," +
				"   ratcomment.targetid," +
				"   ratcomment.cmttime," +
				"   ratcomment.commentor," +
				"   ratcomment.descid " +
				"FROM ratcomment " +
				"WHERE ratcomment.id = " + id;

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				result = new Comment( id );

				result.setModified( false );
				result.setInDatabase( true );

				result.setTarget( rs.getString( col++ ) );
				result.setTargetId( rs.getInt( col++ ) );
				result.setCommentTime( rs.getDate( col++ ) );
				result.setCommentorId( rs.getInt( col++ ) );

				Description desc = null;
				int descId = rs.getInt( col++ );

				if ( descId > 0 )
					{
					desc = Description.getDescription( descId );
					}
				else
					{
					desc = Description.getNewDescription();
					}

				result.setDescription( desc );
				}
			else
				{
				throw new DBIException
					( "no comment found with id '" + id + "'" );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public Comment
	getNewComment( )
		throws DBIException
		{
		Comment result = null;
		String updStr = null;
		int rows;

		try {
			int cmtId = this.getNextId( "comment" );

			updStr =
				"INSERT INTO ratcomment " +
				"  ( id, target, targetid, cmttime, commentor, descid ) " +
				"VALUES" +
				"  ( " + cmtId + ", 'R', '0', 0, 0, 0 )";

			Statement stmt = this.handler.createStatement();
			rows = stmt.executeUpdate( updStr );
			stmt.close();

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not insert comment" );
				}

			result = Description.getComment( cmtId );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + updStr + "'" );
			}

		return result;
		}

	public Vector
	getComments( String target, int targetId )
		throws DBIException
		{
		Vector result = new Vector();
		String queryStr =
			"SELECT ratcomment.id FROM ratcomment " +
			"WHERE  ratcomment.target = '" + target + "' " +
			"  AND  ratcomment.targetid = '" + targetId + "'";

		try {
			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			for ( ; rs.next() ; )
				{
				int col = 1;
				int cmtId = rs.getInt( col++ );

				if ( cmtId > 0 )
					{
					Comment cmt = Description.getComment( cmtId );
					result.addElement( cmt );
					}
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public void
	commit( Description desc )
		throws DBIException
		{
		int descId = desc.getId();

		try {
			StringBuffer buf = new StringBuffer();

			if ( desc.isInDatabase() )
				{
				buf.append( "UPDATE description " );
				buf.append( "SET " );
				buf.append( "  description.url = ?," );
				buf.append( "  description.email = ?," );
				buf.append( "  description.synopsis = ?," );
				buf.append( "  description.mimetype = ?," );
				buf.append( "  description.desctext = ? " );
				buf.append( "WHERE id = '" + descId + "' " );
				}
			else
				{
				buf.append( "INSERT INTO description " );
				buf.append( "( " );
				buf.append( "  description.id," );
				buf.append( "  description.url," );
				buf.append( "  description.email," );
				buf.append( "  description.synopsis," );
				buf.append( "  description.mimetype" );
				buf.append( "  description.desctext " );
				buf.append( ") " );
				buf.append( "VALUES " );
				buf.append( "( ?, ?, ?, ?, ?, ? )" );
				}

			if ( DEBUG )
				{
				System.err.println
					( "DescriptionJDBC.commit: isInDatabase = '"
						+ desc.isInDatabase() + "'  Cmd = '"
						+ buf.toString() + "'" );
				System.err.println
					( "DescriptionJDBC.commit. descId = " + descId );
				}

			Statement stmt = this.handler.createStatement();

			PreparedStatement pS =
				this.handler.createPreparedStatement
					( buf.toString() );
			
			int pidx = 1;
			if ( ! desc.isInDatabase() )
				{
				pS.setInt( pidx++, descId );
				}
		
			this.setNullString
				( pS, pidx++, desc.getURL(), java.sql.Types.CHAR );
			
			if ( desc.getEmail() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, desc.getEmail().getId() );

			pS.setString( pidx++, desc.getSynopsis() );

			this.setNullString
				( pS, pidx++, desc.getMimeType(), java.sql.Types.CHAR );

			this.setLargeString
				( pS, pidx++, desc.getDescription(), java.sql.Types.CHAR );

			int rows = pS.executeUpdate();

			pS.close();

			if ( DEBUG )
				System.err.println
					( "DescriptionJDBC.commit: rows = '" + rows + "'" );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "committing description '" + descId + "'" );
			}
		}

	public void
	commit( EnvDescription envDesc )
		throws DBIException
		{
		int envDescId = envDesc.getId();

		try {
			StringBuffer buf = new StringBuffer();

			if ( envDesc.isInDatabase() )
				{
				buf.append( "UPDATE envdesc " );
				buf.append( "SET " );
				buf.append( "  envdesc.release = ?," );
				buf.append( "  envdesc.envjvm = ?," );
				buf.append( "  envdesc.envos = ?," );
				buf.append( "  envdesc.envosrel = ?," );
				buf.append( "  envdesc.envplat = ?," );
				buf.append( "  envdesc.envdesc = ? " );
				buf.append( "WHERE envdesc.id = '" + envDescId + "' " );
				}
			else
				{
				buf.append( "INSERT INTO envdesc " );
				buf.append( "( " );
				buf.append( "  envdesc.id," );
				buf.append( "  envdesc.release," );
				buf.append( "  envdesc.envjvm," );
				buf.append( "  envdesc.envos," );
				buf.append( "  envdesc.envosrel," );
				buf.append( "  envdesc.envplat," );
				buf.append( "  envdesc.envdesc " );
				buf.append( ") " );
				buf.append( "VALUES " );
				buf.append( "( ?, ?, ?, ?, ?, ?, ? )" );
				}

			if ( DEBUG )
				System.err.println
					( "DescriptionJDBC.commitEnv: isInDatabase = '"
						+ envDesc.isInDatabase() + "'  Cmd = '"
						+ buf.toString() + "'" );

            PreparedStatement pS =
				this.handler.createPreparedStatement
					( buf.toString() );
			
			int pidx = 1;
			if ( ! envDesc.isInDatabase() )
				{
				pS.setInt( pidx++, envDescId );
				}

			this.setNullString
				( pS, pidx++, envDesc.getRelease(), java.sql.Types.CHAR );
			this.setNullString
				( pS, pidx++, envDesc.getJVM(), java.sql.Types.CHAR );
			this.setNullString
				( pS, pidx++, envDesc.getOS(), java.sql.Types.CHAR );
			this.setNullString
				( pS, pidx++, envDesc.getOSRelease(), java.sql.Types.CHAR );
			this.setNullString
				( pS, pidx++, envDesc.getPlatform(), java.sql.Types.CHAR );
			this.setNullString
				( pS, pidx++, envDesc.getDescription(), java.sql.Types.CHAR );

			int rows = pS.executeUpdate();
			
			pS.close();

			if ( DEBUG )
				System.err.println
					( "DescriptionJDBC.commitEnv: rows = '" + rows + "'" );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "committing env description '" + envDescId + "'" );
			}
		}

	public void
	commit( Comment cmt )
		throws DBIException
		{
		int cmtId = cmt.getId();

		try {
			StringBuffer buf = new StringBuffer();

			if ( cmt.getDescription() != null )
				{
				cmt.getDescription().commit();
				}

			if ( cmt.isInDatabase() )
				{
				buf.append( "UPDATE ratcomment " );
				buf.append( "SET " );
				buf.append( "  ratcomment.target = ?," );
				buf.append( "  ratcomment.targetid = ?," );
				buf.append( "  ratcomment.descid = ?," );
				buf.append( "  ratcomment.commentor = ?," );
				buf.append( "  ratcomment.cmttime = ? " );
				buf.append( "WHERE ratcomment.id = '" + cmtId + "' " );
				}
			else
				{
				buf.append( "INSERT INTO ratcomment " );
				buf.append( "( " );
				buf.append( "  ratcomment.id," );
				buf.append( "  ratcomment.target," );
				buf.append( "  ratcomment.targetid," );
				buf.append( "  ratcomment.descid," );
				buf.append( "  ratcomment.commentor," );
				buf.append( "  ratcomment.cmttime " );
				buf.append( ") " );
				buf.append( "VALUES " );
				buf.append( "( ?, ?, ?, ?, ?, ? )" );
				}

			if ( DEBUG )
				System.err.println
					( "DescriptionJDBC.commitComment: isInDatabase = '"
						+ cmt.isInDatabase() + "'  Cmd = '"
						+ buf.toString() + "'" );

            PreparedStatement pS =
				this.handler.createPreparedStatement
					( buf.toString() );
			
			int pidx = 1;
			if ( ! cmt.isInDatabase() )
				{
				pS.setInt( pidx++, cmtId );
				}

			this.setNullString
				( pS, pidx++, cmt.getTarget(), java.sql.Types.CHAR );
			
			pS.setInt( pidx++, cmt.getTargetId() );
			pS.setInt( pidx++, cmt.getDescription().getId() );
			pS.setInt( pidx++, cmt.getCommentorId() );

			java.sql.Timestamp sqlTS =
				new java.sql.Timestamp
					( cmt.getCommentTime().getTime() );

			pS.setTimestamp( pidx++, sqlTS );

			int rows = pS.executeUpdate();
			
			pS.close();

			if ( DEBUG )
				System.err.println
					( "DescriptionJDBC.commitComment: rows = '" + rows + "'" );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "committing env description '" + cmtId + "'" );
			}
		}

	}


