
package org.gjt.bugrat.dbi;

import java.sql.*;
import java.util.*;
import java.net.URL.*;

import org.gjt.bugrat.util.BugRatLogger;


public
class		JDBCHandlerThread
extends		Thread
	{
	/**
	 * This is the SQL statement that we execute in order to check
	 * to see if the JDBC connection is still in place. This depends
	 * on the property 'global.version' being set, which is provided
	 * in the defproperties.sql file.
	 */
	public static final String CONN_CHECK_SQL =
		"SELECT propval from properties " +
		"WHERE  propkey = 'global.version'";

	/**
	 * How often we wake up, in seconds, to see if the checking period
	 * has elapsed.
	 */
	public static final int POLLING_PERIOD_SEC = 7;

	/**
	 * How often, in milliseconds, between connection checks.
	 */
	public static final int CONN_CHECK_PERIOD_MS = (5 * 60 * 1000);

	private boolean			done = false;
	private boolean			debug = false;
	private int				retries = 0;
	private long			lastMillis = 0;
	private String			sqlStr = null;
	private BugRatLogger	logger = null;

	private JDBCHandler		handler = null;


	public
	JDBCHandlerThread( JDBCHandler h, BugRatLogger logger )
		{
		super();

		this.handler = h;
		this.logger = logger;

		this.done = false;
		this.lastMillis = 0;
		this.retries = 0;
		}

	public void
	shutdown()
		{
		this.done = true;
		}

	public void
	run()
		{
		this.lastMillis = System.currentTimeMillis();

		// UNDONE - is there a better way to parameterize this?!
		this.sqlStr = CONN_CHECK_SQL;

		for ( ; ; )
			{
			if ( this.done )
				break;

			long curMillis = System.currentTimeMillis();

			if ( (curMillis - this.lastMillis) > CONN_CHECK_PERIOD_MS )
				{
				// NOTE
				// Set lastMillis here, since checkConnection()
				// might change its value...
				//
				this.lastMillis = curMillis;
				this.checkConnection();
				}
			else
				{
				try { this.sleep( POLLING_PERIOD_SEC ); }
					catch ( InterruptedException ex ) { }
				}
			}
		}

	private void
	checkConnection()
		{
		if ( this.debug )
			System.err.println
				( "JDBCHandlerThread: check connection, is it open? "
					+ this.handler.isOpen() );

		if ( ! this.handler.isOpen() )
			{
			try {
				this.handler.reopenConnection();
				}
			catch ( SQLException exSql )
				{
				if ( this.logger != null ) this.logger.log
					( "JDBCHandlerThread: reopening connection ", exSql );
				}
			}

		try {
			Statement st = this.handler.createStatement();
			ResultSet rs = st.executeQuery( this.sqlStr );
			rs.close();
			st.close();
			this.retries = 0;
			}
		catch ( SQLException ex )
			{
			if ( this.logger != null ) this.logger.log
				( "JDBCHandlerThread: while checking connection ", ex );

			// This will set us up to retry the connection. We use
			// a geometric backoff of 5 seconds for every retry.
			//
			this.retries++;
			this.lastMillis =
				System.currentTimeMillis()
					- ( CONN_CHECK_PERIOD_MS
						- (this.retries * 5 * 1000) );

			this.handler.closeConnection();
			}
		}

	}


