
package org.gjt.bugrat.dbi;

import java.sql.*;
import java.util.Hashtable;
import org.gjt.bugrat.db.*;


public class
PersonJDBC extends BugRatJDBC
	implements PersonDBI
	{
	private static final boolean	DEBUG = false;

	public
	PersonJDBC( JDBCHandler handler )
		{
		super( handler );
		}

	public Person
	getPerson( int id )
		throws DBIException
		{
		Person result = null;
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   person.type," +
				"   person.name," +
				"   person.email," +
				"   person.login," +
				"   person.org," +
				"   person.phone," +
				"   person.fax," +
				"   person.url " +
				"FROM person " +
				"WHERE person.id = '" + id + "'";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				String type = rs.getString( col++ );
				result = new Person( id, type );

				result.setModified( false );
				result.setInDatabase( true );

				result.setName( rs.getString( col++ ) );
				result.setEmail( rs.getString( col++ ) );
				result.setLogin( rs.getString( col++ ) );
				result.setOrganization( this.getString( rs, col++ ) );
				result.setPhone( this.getString( rs, col++ ) );
				result.setFAX( this.getString( rs, col++ ) );
				result.setURL( this.getString( rs, col++ ) );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public Person
	getPerson( String email )
		throws DBIException
		{
		Person result = null;
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   person.id," +
				"   person.type," +
				"   person.name," +
				"   person.login," +
				"   person.org," +
				"   person.phone," +
				"   person.fax," +
				"   person.url " +
				"FROM person " +
				"WHERE person.email = '" + email + "'";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				int id = rs.getInt( col++ );
				String type = rs.getString( col++ );
				result = new Person( id, type );

				result.setModified( false );
				result.setInDatabase( true );

				result.setName( rs.getString( col++ ) );
				result.setEmail( email );
				result.setLogin( rs.getString( col++ ) );
				result.setOrganization( this.getString( rs, col++ ) );
				result.setPhone( this.getString( rs, col++ ) );
				result.setFAX( this.getString( rs, col++ ) );
				result.setURL( this.getString( rs, col++ ) );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public Person
	getLoginPerson( String login )
		throws DBIException
		{
		Person result = null;
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   person.id," +
				"   person.type," +
				"   person.name," +
				"   person.email," +
				"   person.org," +
				"   person.phone," +
				"   person.fax," +
				"   person.url " +
				"FROM person " +
				"WHERE person.login = '" + login + "'";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			if ( rs.next() )
				{
				int col = 1;
				int id = rs.getInt( col++ );
				String type = rs.getString( col++ );
				result = new Person( id, type );

				result.setModified( false );
				result.setInDatabase( true );

				result.setName( rs.getString( col++ ) );
				result.setEmail( rs.getString( col++ ) );
				result.setLogin( login );
				result.setOrganization( this.getString( rs, col++ ) );
				result.setPhone( this.getString( rs, col++ ) );
				result.setFAX( this.getString( rs, col++ ) );
				result.setURL( this.getString( rs, col++ ) );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public Person
	getNewPerson( String type, String email, String name )
		throws DBIException
		{
		Person result = null;
		String queryStr = null;

		try {
			int personId = this.getNextId( "person" );

			queryStr =
				"INSERT INTO person " +
				"  ( id, type, email, name ) " +
				"VALUES ( ?, ?, ?, ? )";

			Statement stmt = this.handler.createStatement();

            PreparedStatement pS =
				this.handler.createPreparedStatement( queryStr );

			int pidx = 1;
			pS.setInt( pidx++, personId );
			pS.setString( pidx++, type );
			pS.setString( pidx++, email );
			pS.setString( pidx++, name );

			int rows = pS.executeUpdate();

			pS.close();

			if ( DEBUG )
				System.err.println( "CREATING NEW PERSON rows=" + rows );

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not create new person" );
				}

			if ( DEBUG )
				System.err.println( "CREATE PERSON id=" + personId );

			result = this.getPerson( personId );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public Hashtable
	getAllPersons()
		throws DBIException
		{
		Hashtable result = new Hashtable();
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   person.id," +
				"   person.type," +
				"   person.name," +
				"   person.email," +
				"   person.login," +
				"   person.org," +
				"   person.phone," +
				"   person.fax," +
				"   person.url " +
				"FROM person ";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			for ( ; rs.next() ; )
				{
				int col = 1;
				int id = rs.getInt( col++ );
				String type = rs.getString( col++ );
				Person p = new Person( id, type );

				p.setModified( false );
				p.setInDatabase( true );

				p.setName( rs.getString( col++ ) );
				p.setEmail( rs.getString( col++ ) );
				p.setLogin( rs.getString( col++ ) );
				p.setOrganization( this.getString( rs, col++ ) );
				p.setPhone( this.getString( rs, col++ ) );
				p.setFAX( this.getString( rs, col++ ) );
				p.setURL( this.getString( rs, col++ ) );
				
				result.put( p.getEmail(), p );
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public void
	commit( Person per )
		throws DBIException
		{
		try {
			StringBuffer buf = new StringBuffer();

			if ( per.isInDatabase() )
				{
				buf.append( "UPDATE person " );
				buf.append( "SET " );
				buf.append( "  person.type = ?," );
				buf.append( "  person.name = ?," );
				buf.append( "  person.email = ?," );
				buf.append( "  person.login = ?," );
				buf.append( "  person.org = ?," );
				buf.append( "  person.phone = ?," );
				buf.append( "  person.fax = ?," );
				buf.append( "  person.url = ? " );
				buf.append( "WHERE" );
				buf.append( "  person.id = '" + per.getId() + "' " );
				}
			else
				{
				buf.append( "INSERT INTO person " );
				buf.append( "( " );
				buf.append( "  person.id," );
				buf.append( "  person.type," );
				buf.append( "  person.name," );
				buf.append( "  person.email," );
				buf.append( "  person.login," );
				buf.append( "  person.org," );
				buf.append( "  person.phone," );
				buf.append( "  person.fax," );
				buf.append( "  person.url " );
				buf.append( ") " );
				buf.append( "VALUES " );
				buf.append( "( ?, ?, ?, ?, ?, ?, ?, ?, ? )" );
				}

			if ( DEBUG )
				System.err.println
					( "PersonJDBC.commit: isInDatabase = '"
						+ per.isInDatabase() + "'  Cmd = '"
						+ buf.toString() + "'" );

            PreparedStatement pStmt =
				this.handler.createPreparedStatement( buf.toString() );

			int pidx = 1;
			if ( ! per.isInDatabase() )
				{
				pStmt.setInt( pidx++, per.getId() );
				}
			
			pStmt.setString( pidx++, per.getType() );
			if ( DEBUG )
				System.err.println
					( "PersonJDBC.commit: TYPE = '" + per.getType() + "'" );
			pStmt.setString( pidx++, per.getName() );
			pStmt.setString( pidx++, per.getEmail() );

			this.setNullString
				( pStmt, pidx++, per.getLogin(), java.sql.Types.CHAR );
			this.setNullString
				( pStmt, pidx++, per.getOrganization(), java.sql.Types.CHAR );
			this.setNullString
				( pStmt, pidx++, per.getPhone(), java.sql.Types.CHAR );
			this.setNullString
				( pStmt, pidx++, per.getFAX(), java.sql.Types.CHAR );
			this.setNullString
				( pStmt, pidx++, per.getURL(), java.sql.Types.CHAR );

			int rows = pStmt.executeUpdate();
			
			pStmt.close();

			if ( DEBUG )
				System.err.println
					( "PersonJDBC.commit: rows = '" + rows + "'" );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "committing person '" + per.getId() + "'" );
			}
		}

	}


