
package org.gjt.bugrat.dbi;

import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.text.FieldPosition;
import java.text.ParsePosition;
import java.util.Vector;
import java.util.Date;
import java.util.Hashtable;

import org.gjt.bugrat.db.*;


public
class		ReportJDBC
extends		BugRatJDBC
implements	ReportDBI
	{
	private static final boolean	DEBUG = false;

	public
	ReportJDBC( JDBCHandler handler )
		{
		super( handler );
		}

	public Report
	getNewReport()
		throws DBIException
		{
		int rows;
		Report result = null;
		String queryStr = null;

		try {
			//
			// CREATE DESCRIPTION RECORD
			//
			int repDescId = this.getNextId( "description" );

			queryStr =
				"INSERT INTO description " +
				"  ( id, synopsis ) " +
				"VALUES " +
				"  ( " + repDescId + ", '' ) ";

			Statement stmt = this.handler.createStatement();
			rows = stmt.executeUpdate( queryStr );
			stmt.close();

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not insert description" );
				}

			//
			// CREATE ENVDESC RECORD
			//
			int envDescId = this.getNextId( "envdesc" );

			queryStr =
				"INSERT INTO envdesc " +
				"  ( id ) " +
				"VALUES " +
				"  ( " + envDescId + " ) ";

			stmt = this.handler.createStatement();
			rows = stmt.executeUpdate( queryStr );
			stmt.close();

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not insert environment description" );
				}

			//
			// CREATE REPORT RECORD
			//
			int reportId = this.getNextId( "report" );

			StringBuffer buf = new StringBuffer( 400 );

			buf.append( "INSERT INTO report " );
			buf.append( " ( " );
			buf.append( "  report.id," );
			buf.append( "  report.source," );
			buf.append( "  report.srcid," );
			buf.append( "  report.submitter," );
			buf.append( "  report.submitted," );
			buf.append( "  report.responsible," );
			buf.append( "  report.project," );
			buf.append( "  report.category," );
			buf.append( "  report.subcat," );
			buf.append( "  report.class," );
			buf.append( "  report.state," );
			buf.append( "  report.severity, ");
			buf.append( "  report.priority,");
			buf.append( "  report.confidence,");
			buf.append( "  report.repdesc,");
			buf.append( "  report.envdesc,");
			buf.append( "  report.repro,");
			buf.append( "  report.around ");
			buf.append( " ) " );
			buf.append( "VALUES " );
			buf.append( "( ?, ?, ?, ?, ?, ?, ?, ?, ?," );
			buf.append( "  ?, ?, ?, ?, ?, ?, ?, ?, ? )" );

			PreparedStatement pS =
				this.handler.createPreparedStatement( buf.toString() );

			int pidx = 1;
			pS.setInt( pidx++, reportId );					// id
			pS.setString( pidx++, Report.SOURCE_WWW );		// source
			pS.setInt( pidx++, 0 );							// srcid
			pS.setInt( pidx++, 0 );							// submitter
			pS.setTimestamp									// submitted
				( pidx++, this.normalizeDate( new Date() ) );
			pS.setInt( pidx++, 0 );							// responsible
			pS.setString( pidx++, "" );						// project
			pS.setString( pidx++, "" );						// category
			pS.setString( pidx++, "" );						// subcat
			pS.setString( pidx++, "" );						// class
			pS.setString( pidx++, "" );						// state
			pS.setString( pidx++, "" );						// severity
			pS.setString( pidx++, "" );						// priority
			pS.setString( pidx++, "" );						// confidence
			pS.setInt( pidx++, repDescId );					// repdesc
			pS.setInt( pidx++, envDescId );					// envdesc
			pS.setNull( pidx++, java.sql.Types.INTEGER );	// repro
			pS.setNull( pidx++, java.sql.Types.INTEGER );	// around

			rows = pS.executeUpdate();

			pS.close();

			if ( rows < 1 )
				{
				throw new DBIException
					( "could not create new report" );
				}

			if ( DEBUG )
				System.err.println
					( "NEW REPORT id=" + reportId
						+ ", descId=" + repDescId
						+ ", envDescId=" + envDescId );

			return this.getReport( reportId );
			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}
		}

	public Report
	getReport( int reportId )
		throws DBIException
		{
		Report result = null;
		String queryStr = null;

		try {
			queryStr = 
				"SELECT" +
				"   report.source," +
				"   report.srcid," +
				"   report.submitter," +
				"   report.submitted," +
				"   report.responsible," +
				"   report.confidence," +
				"   report.severity," +
				"   report.priority," +
				"   report.state," +
				"   report.class," +
				"   report.project," +
				"   report.category," +
				"   report.subcat," +
				"   report.repdesc," +
				"   report.envdesc," +
				"   report.repro," +
				"   report.around " +
				"FROM report " +
				"WHERE report.id = '" + reportId + "'";

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( queryStr );

			int descId = 0;
			int envDescId = 0;
			int reproDescId = 0;
			int aroundDescId = 0;

			if ( rs.next() )
				{
				int col = 1;
				result = new Report( reportId );

				result.setModified( false );
				result.setInDatabase( true );

				result.setSource( rs.getString( col++ ) );
				result.setSourceId( rs.getInt( col++ ) );
				result.setSubmitter( rs.getInt( col++ ) );

				result.setSubmitted
					( this.localizeDate( rs.getTimestamp( col++ ) ) );

				result.setResponsible( rs.getInt( col++ ) );
				result.setConfidence( rs.getString( col++ ) );
				result.setSeverity( rs.getString( col++ ) );
				result.setPriority( rs.getString( col++ ) );
				result.setState( rs.getString( col++ ) );
				result.setReportClass( rs.getString( col++ ) );
				result.setProject( rs.getString( col++ ) );
				result.setCategory( rs.getString( col++ ) );
				result.setSubCategory( rs.getString( col++ ) );

				// Delay getting these until the ResultSet is closed...
				descId = rs.getInt( col++ );
				envDescId = rs.getInt( col++ );
				reproDescId = rs.getInt( col++ );
				aroundDescId = rs.getInt( col++ );
				}

			rs.close();
			stmt.close();

			if ( descId > 0 )
				{
				result.setDescription
					( Description.getDescription( descId ) );
				}

			if ( envDescId > 0 )
				{
				result.setEnvDescription
					( EnvDescription.getEnvDescription( envDescId ) );
				}

			if ( reproDescId > 0 )
				{
				result.setReproDescription
					( Description.getDescription( reproDescId ) );
				}

			if ( aroundDescId > 0 )
				{
				result.setAroundDescription
					( Description.getDescription( aroundDescId ) );
				}


			}
		catch ( SQLException ex )
			{
			if ( DEBUG )
				ex.printStackTrace( System.err );
			throw new DBIException
				( ex, "statement '" + queryStr + "'" );
			}

		return result;
		}

	public Vector
	matchReports( String selectStr )
		throws DBIException
		{
		Vector result = new Vector();

		try {
			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( selectStr );

			for ( ; rs.next() ; )
				{
				int col = 1;
				int repId = rs.getInt( col++ );
				Report rep = this.getReport( repId );
				result.addElement( rep );
				}

			rs.close();
			stmt.close();

			for ( int i = 0, sz = result.size() ; i < sz ; ++i )
				{
				}
			}
		catch ( SQLException ex )
			{
			throw new DBIException
				( ex, "statement '" + selectStr + "'" );
			}

		return result;
		}

	public void
	commit( Report rep )
		throws DBIException
		{
		int repId = rep.getId();

		try {
			StringBuffer buf = new StringBuffer();

			rep.getDescription().commit();

			rep.getEnvDescription().commit();

			if ( rep.getReproDescription() != null )
				rep.getReproDescription().commit();

			if ( rep.getAroundDescription() != null )
				rep.getAroundDescription().commit();

			if ( rep.isInDatabase() )
				{
				buf.append( "UPDATE" );
				buf.append( "  report " );
				buf.append( "SET " );
				buf.append( "  report.source = ?," );
				buf.append( "  report.srcid = ?," );
				buf.append( "  report.submitter = ?," );
				buf.append( "  report.submitted = ?," );
				buf.append( "  report.responsible = ?," );
				buf.append( "  report.project = ?," );
				buf.append( "  report.category = ?," );
				buf.append( "  report.subcat = ?," );
				buf.append( "  report.class = ?," );
				buf.append( "  report.state = ?," );
				buf.append( "  report.severity = ?," );
				buf.append( "  report.priority = ?," );
				buf.append( "  report.confidence = ?," );
				buf.append( "  report.repdesc = ?," );
				buf.append( "  report.envdesc = ?," );
				buf.append( "  report.repro = ?," );
				buf.append( "  report.around = ? " );
				buf.append( "WHERE" );
				buf.append( "  report.id = '" + repId + "' " );
				}
			else
				{
				buf.append( "INSERT INTO report " );
				buf.append( " ( " );
				buf.append( "  report.id," );
				buf.append( "  report.source," );
				buf.append( "  report.srcid," );
				buf.append( "  report.submitter," );
				buf.append( "  report.submitted," );
				buf.append( "  report.responsible," );
				buf.append( "  report.project," );
				buf.append( "  report.category," );
				buf.append( "  report.subcat," );
				buf.append( "  report.class," );
				buf.append( "  report.state," );
				buf.append( "  report.severity, ");
				buf.append( "  report.priority,");
				buf.append( "  report.confidence,");
				buf.append( "  report.repdesc,");
				buf.append( "  report.envdesc,");
				buf.append( "  report.repro,");
				buf.append( "  report.around ");
				buf.append( " ) " );
				buf.append( "VALUES " );
				buf.append( "( ?, ?, ?, ?, ?, ?, ?, ?, ?," );
				buf.append( "  ?, ?, ?, ?, ?, ?, ?, ?, ? )" );
				}

			if ( DEBUG )
				System.err.println
					( "ReportJDBC.commit: isInDatabase = '"
						+ rep.isInDatabase() + "'  Cmd = '"
						+ buf.toString() + "'" );

			PreparedStatement pS =
				this.handler.createPreparedStatement( buf.toString() );

			int pidx = 1;
			if ( ! rep.isInDatabase() )
				{
				pS.setInt( pidx++, repId );
				}

			pS.setString( pidx++, rep.getSource() );
			pS.setInt( pidx++, rep.getSourceId() );
			pS.setInt( pidx++, rep.getSubmitter() );

			pS.setTimestamp
				( pidx++, this.normalizeDate(  rep.getSubmitted() ) );

			pS.setInt( pidx++, rep.getResponsible() );
			pS.setString( pidx++, rep.getProject() );
			pS.setString( pidx++, rep.getCategory() );
			pS.setString( pidx++, rep.getSubCategory() );
			pS.setString( pidx++, rep.getReportClass() );
			pS.setString( pidx++, rep.getState() );
			pS.setString( pidx++, rep.getSeverity() );
			pS.setString( pidx++, rep.getPriority() );
			pS.setString( pidx++, rep.getConfidence() );

			if ( rep.getDescription() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, rep.getDescription().getId() );

			if ( rep.getEnvDescription() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, rep.getEnvDescription().getId() );

			if ( rep.getReproDescription() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, rep.getReproDescription().getId() );

			if ( rep.getAroundDescription() == null )
				pS.setNull( pidx++, java.sql.Types.INTEGER );
			else
				pS.setInt( pidx++, rep.getAroundDescription().getId() );

			int rows = pS.executeUpdate();
			
			pS.close();

			if ( DEBUG )
				System.err.println
					( "ReportJDBC.commit: rows = '" + rows + "'" );
			}
		catch ( SQLException ex )
			{
			throw new DBIException
				( ex, "committing report '" + repId + "'" );
			}
		}

	/**
	*
	*  linkReport - This links a report to a bug
	*
	**/

	public void
	linkReport( int repId, int bugId )
		throws DBIException
		{
		try {
			StringBuffer buf = new StringBuffer();

			buf.append( "INSERT into repbug " );
			buf.append( "  ( report, bug ) " );
			buf.append( "VALUES ( " );
			buf.append( repId + ", " + bugId );
			buf.append( " ) " );

			Statement stmt = this.handler.createStatement();

			int rows = stmt.executeUpdate( buf.toString());

			stmt.close();
			}
		catch ( SQLException ex )
			{
			throw new DBIException
				( ex, "linking report '" + repId
					+ "' to bug '" + bugId + "'" );
			}
		}

	public boolean
	isReportLinkedToBug( int repId, int bugId )
		throws DBIException
		{
		boolean result = false;

		try {
			StringBuffer buf = new StringBuffer();

			buf.append( "SELECT" );
			buf.append( "  repbug.report " );
			buf.append( "FROM repbug " );
			buf.append( "WHERE repbug.report = '" + repId + "' " );
			buf.append( "  AND repbug.bug = '" + bugId + "' " );

			Statement stmt = this.handler.createStatement();
			ResultSet rs = stmt.executeQuery( buf.toString() );

			if ( rs.next() )
				{
				result = true;
				}

			rs.close();
			stmt.close();
			}
		catch ( SQLException ex )
			{
			throw new DBIException
				( ex, "selecting link of report '" + repId
					+ "' to bug '" + bugId + "'" );
			}

		return result;
		}

	}


