
package org.gjt.bugrat.servlet;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;

import org.gjt.bugrat.db.*;
import org.gjt.bugrat.dbi.*;
import org.gjt.bugrat.util.BugRatLogger;


public
class		BugRatDBI
extends		GenericServlet
implements	BugRatConstants, BugRatLogger
	{
	protected String			jdbcUrl = null;
	protected Properties		jdbcProps = null;
	protected String			jdbcDriverClassName = null;

	protected BugDBI			bugDBI = null;
	protected DescriptionDBI	descDBI = null;
	protected PersonDBI			personDBI = null;
	protected ReportDBI			reportDBI = null;
	protected PropertyDBI		propDBI = null;

	protected ServletConfig		config = null;
	protected ServletContext	context = null;


	public void
	init( ServletConfig config )
		throws ServletException
		{
		super.init( config );

		this.config = config;
		this.context = config.getServletContext();

		this.log( "BugRatDBI: INIT" );

		Enumeration enum = config.getInitParameterNames();
		for ( ; enum.hasMoreElements() ; )
			{
			String name = (String) enum.nextElement();
			this.log( "BugRatDBI: CONFIG " + name
				+ " = '" + config.getInitParameter( name ) + "'" );
			}

		String val;
		val = config.getInitParameter( "jdbcUrl" );
		if ( val != null )
			this.jdbcUrl = val;

		this.log( "  JDBC URL:" );
		this.log( "    Url = '" + this.jdbcUrl + "'" );

		this.jdbcProps = new Properties();
		for ( int i = 1 ; ; ++i )
			{
			val = config.getInitParameter( "jdbcProp_" + i );
			if ( val == null )
				break;

			int index = val.indexOf( "=" );
			if ( index > 0 )
				{
				String propKey = val.substring( 0, index );
				String propVal = val.substring( index + 1);
				this.log( "    Props '" + propKey + "=" + propVal + "'" );
				this.jdbcProps.put( propKey, propVal );
				}
			}

		val = config.getInitParameter( "jdbcDriverClassName" );
		if ( val != null )
			this.jdbcDriverClassName = val;

		this.log( "  JDBC Driver:" );
		this.log( "    Driver  = '" + this.jdbcDriverClassName + "'" );

		try {
			// NOTE You MUST call the JDBC Parameters and
			//      Driver methods BEFORE call setCurrentDBIManager().
			DBIManager dbiMgr = DBIManager.getInstance();

			dbiMgr.setJDBCParameters
				( this.jdbcUrl, this.jdbcDriverClassName, this.jdbcProps, this );

			dbiMgr.setCurrentDBIManager( DBIManager.JDBC );

			this.personDBI = dbiMgr.getPersonDBI();
			this.reportDBI = dbiMgr.getReportDBI();
			this.propDBI = dbiMgr.getPropertyDBI();
			this.descDBI = dbiMgr.getDescriptionDBI();
			this.bugDBI = dbiMgr.getBugDBI();
			}
		catch ( DBIException ex )
			{
			System.err.println
				( "Could not establish DBIManager." );
			ex.printStackTrace( System.err );
			throw new ServletException
				( "could not set DBIManager to JDBC." );
			}
		}

	public void
	service( ServletRequest req, ServletResponse res )
		throws ServletException, IOException
		{
		String resp = null;

		res.setContentType( "application/octet-stream" );

		ObjectOutputStream outObj =
			new ObjectOutputStream(
				new BufferedOutputStream(
					res.getOutputStream() ) );

		ObjectInputStream in =
			new ObjectInputStream(
				new BufferedInputStream(
					req.getInputStream() ) );

		try {
			JSRVRequest request =
				(JSRVRequest) in.readObject();

			JSRVResponse response =
				new JSRVResponse
					( request.getRequestId(), 0, "ok" );

			System.err.println
				("BugRatDBI: request=" + request.getRequest()
					+ ", id=" + request.getRequestId() );

			switch ( request.getRequest() )
				{
				case JSRVConstants.REQ_NOOP:
					{
					System.err.println( "REQ_NOOP" );
					response.setSuccess( true );
					}
					break;

				case JSRVConstants.REQ_GET_BUG:
					{
					int bugId =
						((Integer) request.getParameter
							( JSRVConstants.P_BUG_ID )).intValue();

					try {
						Bug bug = Bug.getBug( bugId );
						response.setSuccess( bug != null );
						response.setResult( JSRVConstants.P_BUG, bug );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_BUG" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_NEW_BUG:
					{
					try {
						Bug bug = Bug.getNewBug();
						response.setSuccess( bug != null );
						response.setResult( JSRVConstants.P_BUG, bug );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_NEW_BUG" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_MATCH_BUGS:
					{
					String matchExpr = (String)
						request.getParameter
							( JSRVConstants.P_MATCH_EXPR );

					System.err.println
						( "MATCH_BUGS: matchExpr = '" + matchExpr + "'" );

					try {
						Vector result = Bug.matchBugs( matchExpr );
						response.setSuccess( result != null );
						response.setResult
							( JSRVConstants.P_BUG_VECTOR, result );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_MATCH_BUGS" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_REPORT:
					{
					int repId =
						((Integer) request.getParameter
							( JSRVConstants.P_REPORT_ID )).intValue();

					try {
						Report rep = Report.getReport( repId );
						response.setSuccess( rep != null );
						response.setResult( JSRVConstants.P_REPORT, rep );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_REPORT" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_NEW_REPORT:
					{
					try {
						Report rep = Report.getNewReport();
						response.setSuccess( rep != null );
						response.setResult( JSRVConstants.P_REPORT, rep );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_NEW_REPORT" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_MATCH_REPORTS:
					{
					String matchExpr = (String)
						request.getParameter
							( JSRVConstants.P_MATCH_EXPR );

					System.err.println
						( "MATCH_REPORTS: matchExpr = '" + matchExpr + "'" );

					try {
						Vector result = Report.matchReports( matchExpr );
						response.setSuccess( result != null );
						response.setResult
							( JSRVConstants.P_REPORT_VECTOR, result );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_MATCH_REPORTS" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_PERSON:
					{
					int personId =
						((Integer) request.getParameter
							( JSRVConstants.P_PERSON_ID )).intValue();

					System.err.println
						( "GET_PERSON: personId = '" + personId + "'" );

					try {
						Person per = Person.getPerson( personId );
						response.setSuccess( per != null );
						response.setResult( JSRVConstants.P_PERSON, per );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_PERSON" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_COMMIT_PERSON:
					{
					Person p = (Person)
						request.getParameter
							( JSRVConstants.P_PERSON );

					System.err.println
						( "COMMIT_PERSON: Id = '" + p.getId() + "'" );

					try {
						this.personDBI.commit( p );
						response.setSuccess( true );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_COMMIT_PERSON" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_ALLPERSONS:
					{
					System.err.println
						( "GET_ALLPERSONS: " );

					try {
						Hashtable h = Person.getAllPersons();
						response.setSuccess( h != null );
						response.setResult
							( JSRVConstants.P_PERSON_HASHTABLE, h );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_ALLPERSONS" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_ALLCATEGORIES:
					{
					System.err.println
						( "GET_ALLCATEGORIES: " );

					try {
						Vector cats = Category.getAllCategories();
						response.setSuccess( cats != null );
						response.setResult
							( JSRVConstants.P_CATEGORY_VECTOR, cats );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_ALLCATEGORIES" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_ALLLEVELPARAMS:
					{
					String table = (String)
						request.getParameter
							( JSRVConstants.P_LEVEL_TABLE_ID );

					System.err.println
						( "GET_ALLLEVELPARAMS: table = '" + table + "'" );

					try {
						Vector lpV =
							LevelParam.getAllLevelParams( table );
						response.setSuccess( lpV != null );
						response.setResult
							( JSRVConstants.P_LEVEL_PARAM_VECTOR, lpV );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_ALLLEVELPARAMS" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_ALLPROPERTIES:
					{
					System.err.println
						( "GET_ALLPROPERTIES: " );

					try {
						Hashtable h = BRATProperty.getAllProperties();
						response.setSuccess( h != null );
						response.setResult
							( JSRVConstants.P_PROPERTY_HASHTABLE, h );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_GET_ALLPROPERTIES" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_COMMIT_PROPERTY:
					{
					BRATProperty bp = (BRATProperty)
						request.getParameter
							( JSRVConstants.P_PROPERTY );

					System.err.println
						( "COMMIT_PROPERTY: Key = '" + bp.getKey() + "'" );

					try {
						this.propDBI.commit( bp );
						response.setSuccess( true );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						// UNDONE - Do we want to send back the exception?!!?
						System.err.println( "REQ_COMMIT_PROPERTY" );
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_LINK_REPORT:
					{
					System.err.println( "REQ_LINK_REPORT: ");

					Integer reportId = ( Integer )
						request.getParameter
							( JSRVConstants.P_REPORT_ID );
					Integer bugId = ( Integer )
						request.getParameter
							( JSRVConstants.P_BUG_ID );

					try {
						this.reportDBI.linkReport( 
							reportId.intValue(),
							bugId.intValue() );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						System.err.println(
							"Error linking report to bug.");
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_DESCRIPTION:
					{
					System.err.println( "REQ_GET_DESCRIPTION: ");

					Integer descId = ( Integer )
						request.getParameter
							( JSRVConstants.P_DESCRIPTION_ID );

					try {
						Description desc =
							this.descDBI.getDescription( descId.intValue() );
						response.setSuccess( desc != null );
						response.setResult
							( JSRVConstants.P_DESCRIPTION, desc );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						System.err.println(
							"Error linking report to bug.");
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_NEW_DESCRIPTION:
					{
					System.err.println( "REQ_NEW_DESCRIPTION: ");

					try {
						Description desc =
							this.descDBI.getNewDescription();
						response.setSuccess( desc != null );
						response.setResult( JSRVConstants.P_DESCRIPTION, desc );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						System.err.println
							( "Error creating new environment description.");
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_GET_ENVDESC:
					{
					System.err.println( "REQ_GET_ENVDESC: ");

					Integer descId = ( Integer )
						request.getParameter
							( JSRVConstants.P_ENVDESC_ID );

					try {
						EnvDescription desc =
							this.descDBI.getEnvDescription( descId.intValue() );
						response.setSuccess( desc != null );
						response.setResult( JSRVConstants.P_ENVDESC, desc );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						System.err.println
							( "Error retrieving environment description.");
						ex.printStackTrace( System.err );
						}
					}
					break;

				case JSRVConstants.REQ_NEW_ENVDESC:
					{
					System.err.println( "REQ_NEW_ENVDESC: ");

					try {
						EnvDescription desc =
							this.descDBI.getNewEnvDescription();
						response.setSuccess( desc != null );
						response.setResult( JSRVConstants.P_ENVDESC, desc );
						}
					catch ( DBIException ex )
						{
						response.setSuccess( false );
						System.err.println
							( "Error creating new environment description.");
						ex.printStackTrace( System.err );
						}
					}
					break;

				default:
					System.err.println
						( "BugRatDBI: UNKNOWN REQUEST id = "
							+ request.getRequest() );

					response.setSuccess( false );
					response.setErrorCode( -1 );
					response.setErrorMessage
						( "Unknown Request ID = "
							+ request.getRequest() );
					break;
				}

			outObj.writeObject( response );

			in.close();
			outObj.close();
			}
		catch ( IOException ex )
			{
			System.err.println( "IOException" );
			ex.printStackTrace( System.err );
			}
		catch ( ClassNotFoundException ex )
			{
			System.err.println( "Class Not Found" );
			ex.printStackTrace( System.err );
			}
		}

	}

