
package org.gjt.bugrat.servlet;

import java.io.*;
import java.net.*;


public class
JSRVConnection extends Object
	implements Runnable
	{
	int			apiVersion;

	URL			url;
	String		user = null;
	String		pass = null;
	String		encrypt = null;


	public
	JSRVConnection( String urlSpec )
		throws JSRVException
		{
		try { this.url = new URL( urlSpec ); }
		catch ( MalformedURLException ex )
			{
			throw new JSRVException
				( "Jart Servlet bad url '" + urlSpec + "', " + ex.getMessage() );
			}
		}

	public
	JSRVConnection( String urlSpec, String user, String pass )
		throws JSRVException
		{
		this( urlSpec );
		this.setLoginInfo( user, pass );
		}

	public
	JSRVConnection( URL url )
		{
		this.url = url;
		}

	public
	JSRVConnection( URL url, String user, String pass )
		{
		this.url = url;
		this.setLoginInfo( user, pass );
		}

	public void
	setLoginInfo( String user, String pass )
		{
		// UNDONE - HOW DO WE DEAL WITH "NO PASSWORD"?!?!
		this.user = user;
		this.pass = pass;
		this.encrypt = null;
		if ( this.user != null && this.pass != null )
			{
			this.encrypt =
				this.encodeAuth( this.user, this.pass );
			}
		}

	public JSRVResponse
	performRequest( JSRVRequest request )
		throws JSRVException
		{
		JSRVResponse result = null;
		HttpURLConnection conn = null;

		String where = "BEGIN";

		try {
			where = "opening";
			conn = (HttpURLConnection) this.url.openConnection();

			where = "configuring";
			conn.setDoInput( true );
			conn.setDoOutput( true );
			conn.setAllowUserInteraction( true );
			conn.setRequestMethod( "POST" );

			conn.setRequestProperty
				( "Content-type",
					"application/octet-stream" );

			if ( this.encrypt != null )
				{
				conn.setRequestProperty
					( "Authorization", "Basic " + this.encrypt );
				}

			where = "connecting";
			conn.connect();

			where = "get output stream";
			OutputStream out = conn.getOutputStream();

			BufferedOutputStream bos =
				new BufferedOutputStream( out );

			ObjectOutputStream objout =
				new ObjectOutputStream( bos );

			where = "writing object";
			objout.writeObject( request );

			where = "close output";
			objout.close();

			where = "get input stream";
			InputStream in = conn.getInputStream();

			BufferedInputStream bin =
				new BufferedInputStream( in );

			ObjectInputStream objin =
				new ObjectInputStream( bin );

			where = "read object";
			result = (JSRVResponse) objin.readObject();

			where = "close input";
			objin.close();
			}
		catch ( IOException ex )
			{
			try {
				if ( conn.getResponseCode()
						== HttpURLConnection.HTTP_UNAUTHORIZED )
					{
					throw new JSRVException
						( "Login of user '" + this.user + "' failed.\n",
							conn.getResponseCode(),
							conn.getResponseMessage() );
					}
				else
					{
					throw new JSRVException
						( "IOException: (" + where + ") "
							+ ex.getMessage() );
					}
				}
			catch ( IOException ex2 )
				{
				throw new JSRVException
					( "IOException: (" + where + ") "
						+ ex.getMessage() );
				}
			}
		catch ( ClassNotFoundException ex )
			{
			throw new JSRVException
				( "ClassNotFoundException: ("
					+ where + ") " + ex.getMessage() );
			}

		return result;
		}

	public JSRVResponse
	performAsyncRequest( JSRVRequest request )
		{
		return null;
		}

	public void
	run()
		{
		}

	private static final char		pad64Char = '=';
	private static final String		base64CharSet =
										"ABCDEFGHIJKLMNOPQRSTUVWXYZ" +
										"abcdefghijklmnopqrstuvwxyz" +
										"0123456789+/";

	private String
	encodeAuth( String name, String pass )
		{
		String authStr = name + ":" + pass;
		StringBuffer result = new StringBuffer();

		int state = 0;
		int pos = 0;
		byte nib = 0;
		byte lastNib = 0;
		int len = authStr.length();
		for ( int i = 0 ; i < len || state == 3 ; )
			{
			if ( state != 3 )
				nib = (byte) authStr.charAt(i++);

			switch ( state )
				{
				case 0:
					pos = (int) ( (nib >> 2) & 0x3F );
					state = 1;
					break;

				case 1:
					pos = (int)
						((lastNib & 0x03) << 4) | ( (nib >> 4) & 0x0F );
					state = 2;
					break;

				case 2:
					pos = (int)
						((lastNib & 0x0F) << 2) | ( (nib >> 6) & 0x03 );
					state = 3;
					break;

				case 3:
					pos = (int) ( lastNib & 0x3F );
					state = 0;
					break;
				}

			result.append( base64CharSet.charAt(pos) );

			lastNib = nib;
			}

		if ( state == 1 )
			{
			pos = (int) ((lastNib & 0x03) << 4);
			result.append( base64CharSet.charAt(pos) );
			result.append( pad64Char );
			result.append( pad64Char );
			}
		else if ( state == 2 )
			{
			pos = (int) ((lastNib & 0x0F) << 2);
			result.append( base64CharSet.charAt(pos) );
			result.append( pad64Char );
			}

		return result.toString();
		}

	}

