#
# PROPERTIES
#

#
# G L O B A L    P R O P E R T I E S
#

#
# NOTE this property is used by org.gjt.bugrat.dbi.JDBCHandlerThread
#      as an sql statement that it can successfully execute in
#      order to verify that the JDBC connection is still valid.
#      If you remove this property, you must change the class
#      field 'CONN_CHECK_SQL' in JDBCHandlerThread to be a valid
#      sql statement given your database. For example, in MySQL
#      you can use "SELECT version()".
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'global.version', '2.5.3' );

#
# This is the default person responsible for incoming bug reports.
# The value of zero leaves the report "unassigned".
# Typically, this is set to someone capable of "assigning" the
# report to the correct person.
#
# See BugRatReport.java.processReportPOST(), line 970, where the
# call to report.setResponsible() is made.
#
# You can also define "per project" default responsible persons
# by appending ".projectName" to the property key. So, for
# example, the project named 'bugrat' would use the proprerty
# named 'global.defaultResponsible.bugrat'.
#

INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'global.defaultResponsible', '1' );

#
# This is the default report state for new reports.
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'global.defaultReportState', 'R' );

#
# This is the default confidence level for new reports.
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'global.defaultReportConfidence', 'P' );

#
# This is the default date format string.
# See java.text.SimpleDateFormat for more details.
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'commonServlet.dateFormat', 'MMM d yyyy, hh:mm:ss z' );


#
# The BugRat Administrator email address. This is used to toss
# email about any problems in the code, as well as a default
# address anytime a required address is not available.
#
# You should set this to the address of the person administering
# BugRat for your organization.
#

INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'global.adminAddress', 'root' );

#
# S Q L    P R O P E R T I E S
#
# NOTE - THESE PROPERTIES ARE REQUIRED!
#

#
# This is the SQL used to match a list of reports for the
# ShowLinked and ShowUnlinked reports commands.
# {0} = the project id.
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'sql.showreports.linked',
     "SELECT DISTINCT report.id
      FROM report, repbug
      WHERE report.project = ''{0}''
      AND report.id = repbug.report" );

#
# This statement is used to get the list of bug reports
# in a given project (specified by {0}) that are not linked
# to any bugs. Better dbs might use a sub-query instead of
# the LEFT JOIN provided by MySQL
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'sql.showreports.unlinked',
     "SELECT DISTINCT report.id
      FROM report
      LEFT JOIN repbug
	  ON report.id = repbug.report
      WHERE repbug.report is NULL
      AND report.project = ''{0}''" );

#
# NOTE - END OF 'THESE PROPERTIES ARE REQUIRED!'
#

# Background color for Report Form
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.bgColor', '#E0E0E0' );

# Background color for Report Form Header (title and logo area)
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.hdrColor', '#E0E0E0' );

# Background color for Report Form "Required Fields" area
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.reqColor', '#B0E0B0' );

# Background color for report of success
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.okColor', '#B0E0B0' );

# Background color for report of error
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.errColor', '#E0B0B0' );

# This is the HTML used for the title at the top of the form
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.title',
     '<font size="+2"><strong>BugRat Report Form</strong></font>' );

#
# This is the name of the logo image at the top of the form
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.logoURL', 'therat.gif' );

#
# This the page that will be displayed when the logo is clicked.
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.ratURL', 'Help.html' );

#
# If true, allow comments to be added to bugs in the admin servlet.
#
INSERT INTO properties
	( propkey, propval )
VALUES
	( 'commonServlet.allowsBugComments', 'true' );

#
# If true, allow comments to be added to report in any servlet.
#
INSERT INTO properties
	( propkey, propval )
VALUES
	( 'commonServlet.allowsReportComments', 'true' );

#
# If true, requires the admin servlet to add a bug report comment.
#
INSERT INTO properties
	( propkey, propval )
VALUES
	( 'commonServlet.commentRequiresAdmin', 'false' );

#
# If true, requires the admin servlet to create a new user account (person).
#
INSERT INTO properties
	( propkey, propval )
VALUES
	( 'commonServlet.newPersonRequiresAdmin', 'true' );


#
# This is the HTML for the help page.
#
# {0} ServletContext
# {1} AdminServletName
# {2} ReportServletName
# {3} ViewerServletName
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'reportServlet.helpHtml', '
<h2>BugRat Report Help</h2>

The BugRat Report Form is used to report bugs or other information
about applications, packages, web content, and other software. You
fill in the required fields of the form, add any additional optional
information, and submit the report. Once the report is submitted, it
will be assigned to a reponsible developer who will process your report
from that point on.

<p>
<strong>Note</strong> When you submit a report, you will be given a
<em>report ID number.</em> While it is possible to later locate your
report without this number, you will be able to access all information
regarding your report directly using the ID number. Thus, it is recommended
that you keep a record of the IDs of reports that are important to you.

<p>
To submit your report, you need to fill in the <em>required</em> fields.
These are all located at the top of the page, and are highlighted by
their own background color (typically green). The other fields are
optional, and are not required to file your report. However, it is
strongly recommended that any and all information related to your
report should be included.

<p>
Once you are satisfied with the content of your report, click the
<em>Submit Report</em> button to submit your report. You will then
be shown a confirmation page indicating your report ID number. Please
record the id number for future reference.

<p>
<h3>Report Fields</h3>
<a href="HelpFields.html">
More information about the report input fields
</a>
is also available.

<p>
<h3>Home</h3>
<a href="http://www.gjt.org/pkg/bugrat/">
The BugRat home page
</a>
is the place to look for information about the rat, including new
releases.
' );

#
# This is the HTML for the 'fields' help page.
#
# {0} ServletContext
# {1} AdminServletName
# {2} ReportServletName
# {3} ViewerServletName
#
INSERT INTO properties
	( propkey, propval )
VALUES
	( 'reportServlet.fieldHelpHtml', '
<h2>BugRAT Report Fields Help</h2>

The report consists of fields that are required, and those that
are not. Those that are required are highlighted by a background
color (typically green).

<p>
Here is the list of input fields, and a brief description of what
each field means:

<ul>

<li><strong>Project / Category / SubCategory</strong><br>
The project, category, and subcategory are all specified by selecting
one of the rows in the list. If you already have a project and category
selected, then you will only be presented with the applicable subcategories.

</ul>
' );

#
# This is the HTML for default servlet index pages.
#
#
# {0} ServletContext
# {1} AdminServletName
# {2} ReportServletName
# {3} ViewerServletName
#

INSERT INTO properties
	( propkey, propval )
VALUES
	( 'viewServlet.indexHtml',
		'<h2>BugRat Viewer Servlet</h2>
		These pages are designed to let you browse and search
		the BugRat database for bug reports and bugs. Use the
		list of functions below to link to the function you
		wish to perform.
		<p>

	<font size="+1"><strong>Bug Reports</strong></font>

		<ul>

		<li>
			<a href="{0}/{2}/ReportForm">
			Report a bug using the online report page.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/ChooseReport">
			View a specific bug report.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/ShowProjectAllReports">
			Display all bug reports in a specific project.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/SelectReportsBySubmitter">
			Search for all bug reports submitted by specified person.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/ShowAllReports">
			Display all bug reports for all projects.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/ShowAllLinkedReports">
			Display all bug reports linked to a bug for all projects.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/BrowseReports">
			Browse the bug report database.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/SearchReports">
			Search the bug report database.
			</a>
		</li>

		</ul>


	<font size="+1"><strong>Bugs</strong></font>

		<ul>

		<li>
			<a href="{0}/{3}/ChooseBug">
			View a specific bug by ID.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/ShowAllBugs">
			Display all bugs for all projects.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/ShowProjectAllBugs">
			Display all bugs in a specific project.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/BrowseBugs">
			Browse the bug database.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/SearchBugs">
			Search the bug database.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/SelectBugsByResponsible">
			Search for bugs by the person responsible.
			</a>
		</li>

		</ul>


	<font size="+1"><strong>Miscellaneous</strong></font>

		<ul>

		<li>
			<a href="{0}/{3}/ListPeople">
			List people known to BugRat.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/ListCategories">
			List bug categories for a specific project.
			</a>
		</li>
		<li>
			<a href="{0}/{3}/ShowStatistics">
			Show BugRat database statistics.
			</a>
		</li>

		</ul>

' );

#
# This is the HTML for the admin servlet index page.
#
#
# {0} ServletContext
# {1} AdminServletName
# {2} ReportServletName
# {3} ViewerServletName
#

INSERT INTO properties
	( propkey, propval )
VALUES
	( 'adminServlet.indexHtml',
		'<h2>BugRat Administration Servlet</h2>

		<p>

	<font size="+1"><strong>Bug Reports</strong></font>

		<ul>

		<li>
			<a href="{0}/{1}/LinkReport">
			Link a bug report to a new or existing bug.
			</a>
		</li>
		<p>
			<ul>

			<li>
				<a href="{0}/{2}/ReportForm">
				Report a bug using the online report page.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/ChooseReport">
				View a specific bug report.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/ShowProjectAllReports">
				Display all bug reports in a specific project.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/SelectReportsBySubmitter">
				Search for all bug reports submitted by specified person.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/ShowAllReports">
				Display all bug reports for all projects.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/ShowAllLinkedReports">
				Display all bug reports linked to a bug for all projects.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/BrowseReports">
				Browse the bug report database.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/SearchReports">
				Search the bug report database.
				</a>
			</li>

			</ul>

		</ul>

	<font size="+1"><strong>Bugs</strong></font>

		<ul>

		<li>
			<a href="{0}/{1}/EditBug">
			Modify the state or information about an existing bug.
			</a>
		</li>
		<p>
			<ul>

			<li>
				<a href="{0}/{1}/ChooseBug">
				View a specific bug by ID.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/ShowAllBugs">
				Display all bugs for all projects.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/ShowProjectAllBugs">
				Display all bugs in a specific project.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/BrowseBugs">
				Browse the bug database.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/SearchBugs">
				Search the bug database.
				</a>
			</li>
			<li>
				<a href="{0}/{1}/SelectBugsByResponsible">
				Search for bugs by the person responsible.
				</a>
			</li>

			</ul>

		</ul>


	<font size="+1"><strong>Miscellaneous</strong></font>

		<ul>

		<li>
			<a href="{0}/{1}/ListPeople">
			List people known to BugRat.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/EditPerson/0">
			Add new person.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/EditPerson">
			Edit Person Information.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/ListCategories">
			List all projects and categories.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/ShowStatistics">
			Show BugRat database statistics.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/MailQueueInfo">
			Show mail queue information.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/EditProperty">
			List properties to edit.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/ReloadConfig">
			Reload the servlet properties from the database.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/ShowIdInfo">
			Show the sequence ids used for new object IDs.
			</a>
		</li>
		<li>
			<a href="{0}/{1}/AdjustAllIds">
			Adjust the sequence ids used for new object IDs.
			</a>
		</li>

		</ul>

' );

#
# BugRat Servlet Common Properties ( prefix 'servletCommon.' )
#

#
# The HTML that is the equivalent of the HEAD for each page.
#
# {0} ServletContext
# {1} AdminServletName
# {2} ReportServletName
# {3} ViewerServletName
# {4} title
#
INSERT INTO properties
	( propkey, propval )
VALUES
	( 'servletCommon.htmlHead',
		'
		<html>
		<head>
			<title>
			{4}
			</title>
		</head>
' );

#
# The HTML that is prepended to the BODY for each page of output.
#
# {0} ServletContext
# {1} AdminServletName
# {2} ReportServletName
# {3} ViewerServletName
#
INSERT INTO properties
	( propkey, propval )
VALUES
	( 'servletCommon.htmlBody',
		'
		<body>
' );

#
# The HTML that is appended to every page of output.
#
# {0} ServletContext
# {1} AdminServletName
# {2} ReportServletName
# {3} ViewerServletName
#
INSERT INTO properties
	( propkey, propval )
VALUES
	( 'servletCommon.htmlTail',
		'
		<hr>
		<center>
		<a HREF="{0}/{1}">
		[&nbsp;admin&nbsp;]
		</a>
		<a HREF="{0}/{2}">
		[&nbsp;report&nbsp;]
		</a>
		<a HREF="{0}/{3}">
		[&nbsp;viewer&nbsp;]
		</a>
		</center>
		</body>
		</html>
' );


#
# BugRat Mailer Properties ( prefix 'bratm.' )
#

INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'bratm.robotAddress', 'bugreport@gjt.org' );

INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'bratm.adminAddress', 'bugadmin@gjt.org' );


#
# Help text emailed to the mail filter users.
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'bratm.helpIntro',
'This is an automated message from the Bug Report Mail Daemon.

You received this message because you sent email to the automated
bug reporting address, but your email could not be processed.

If you did not intend to report a bug, then you may disregard this
message.

If you intended to report a bug, then the remainder of this message
will attempt to assist you in generating an email message that can
be processed correctly by the bug reporting system.

Sincerely,
The BugRat.

' );

#
# More help text emailed to the mail filter users.
#
INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'bratm.helpInstruct',
'Bug reports that are sent via email are processed automatically
by a Java program. Because computer programs are limited in their
ability to parse free form text, it is important that your email
follow a precise format.

All bug reports must include certain required information in order
to be processed. In addition to the required information, there is
certain information that may be provided, but is not required.

All of the required fields for a BugRat bug report are single
line fields. The required field are:
   . Project
   . Category
   . SubCategory
   . Class
   . Confidence
   . Priority
   . Severity
   . Release
   . OS
   . OSRelease
   . Platform
   . Synopsis

The optional information for a BugRat bug report is:
   . Description                (multi-line)
   . Environment Description    (multi-line)
   . Reproducible Instructions  (multi-line)
   . Work Around Description    (multi-line)

Each of the above "fields" are included in your email in a
format as follows:

   > FieldName: data

Fields which can be multi-line are expressed as follows:

   > FieldName: line1
   line2
   line3

The multi-line fields are ended by the end of the message,
or by the beginning of a new field. At the end of this
message, you will find a complete list of all of the possible
values that you can use in the report fields. Please note that
the Category field depends on the selected Project, and the
and the SubCategory depends on the selected Category. The most
common error in email bug reports is an improperly selected or
a missing category or subcategory.

All blank lines are ignored, except in multi-line fields.

Here is an example. You could copy this example, paste it into
your email composer, modify the fields with values from the
listings included below, and you should have a valid report.

====================  START OF EXAMPLE  ====================

> Project: bugrat
> Category: email
> SubCategory: report
> Class: swbug
> Confidence: public
> Priority: medium
> Severity: serious
> Release: 1.0
> OS: WinNt
> OSRelease: 4sp3
> Platform: Intel
> Synopsis: Email bug reports fail
> Description: When I send in a bug report via the bugrat
email bug report system, I get nothing but error replies.
I do not get any help reply, but an error report that the
servlet "org.gjt.brat.BugRatReport" is not found.

====================  END   OF EXAMPLE  ====================

' );

#
# BugRat Recognized HTML Tags
#

INSERT INTO properties
   ( propkey, propval )
VALUES
   ( 'servletCommon.htmlTags',
		'a,b,center,dl,dd,dt,em,font,h1,h2,h3,h4,h5,h6,hr,href,i,img,input,li,option,p,pre,select,span,strong,style,table,td,textarea,th,tr,ul' );


