
CREATE TABLE
	sequence
	(
		name			char(64)		NOT NULL,
		nextseq			int				NOT NULL,

	PRIMARY KEY ( name )
	);

CREATE TABLE
	report
	(
		id				int				NOT NULL,
		source			char(1)			NOT NULL,
		srcid			int				NOT NULL,
		submitted		datetime		NOT NULL,
		submitter		int				NOT NULL,
		responsible		int				NOT NULL,
		confidence		char(1)			NOT NULL,
		severity		char(1)			NOT NULL,
		priority		char(1)			NOT NULL,
		state			char(1)			NOT NULL,
		class			char(1)			NOT NULL,
		project			char(8)			NOT NULL,
		category		char(8)			NOT NULL,
		subcat			char(8)			NOT NULL,
		repdesc			int				NOT NULL,
		envdesc			int				NOT NULL,
		repro			int				NULL,
		around			int				NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	bug
	(
		id				int				NOT NULL,
		source			char(1)			NOT NULL,
		srcid			int				NOT NULL,
		responsible		int				NOT NULL,
		opened			datetime		NOT NULL,
		closed			datetime		NULL,
		confidence		char(1)			NOT NULL,
		severity		char(1)			NOT NULL,
		priority		char(1)			NOT NULL,
		state			char(1)			NOT NULL,
		class			char(1)			NOT NULL,
		project			char(8)			NOT NULL,
		category		char(8)			NOT NULL,
		subcat			char(8)			NOT NULL,
		bugdesc			int				NOT NULL,
		envdesc			int				NOT NULL,
		repro			int				NULL,
		around			int				NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	action
	(
		id				int				NOT NULL,
		acttype			char(1)			NOT NULL,
		actor			int				NOT NULL,
		acttime			datetime		NOT NULL,
		target			char(1)			NOT NULL,
		targetid		int				NOT NULL,
		argone			int				NOT NULL,
		argtwo			int				NOT NULL,
		comment			varchar(255)	NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	actiontype
	(
		id				char(1)			NOT NULL,
		name			char(32)		NOT NULL,
		desctext		char(255)		NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	description
	(
		id				int				NOT NULL,
		synopsis		varchar(255)	NOT NULL,
		url				varchar(255)	NULL,
		email			int				NULL,
		mimetype		varchar(128)	NULL,
		desctext		mediumblob		NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	envdesc
	(
		id				int				NOT NULL,
		release			char(32)		NULL,
		envjvm			char(32)		NULL,
		envos			char(32)		NULL,
		envosrel		char(32)		NULL,
		envplat			char(32)		NULL,
		envdesc			char(255)		NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	comment
	(
		id				int				NOT NULL,
		target			char(1)			NOT NULL,
		targetid		int				NOT NULL,
		descid			int				NOT NULL,
		commentor		int				NOT NULL,
		cmttime			datetime		NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	person
	(
		id				int				NOT NULL,
		type			char(1)			NOT NULL,
		email			varchar(128)	NOT NULL,
		name			varchar(128)	NOT NULL,
		login			char(16)		NULL,
		org				varchar(128)	NULL,
		url				varchar(255)	NULL,
		phone			varchar(32)		NULL,
		fax				varchar(32)		NULL,

	PRIMARY KEY( id ),
	UNIQUE emailindex ( email(32) )
	);

CREATE TABLE
	subscribe
	(
		id				int				NOT NULL,
		subtype			char(1)			NOT NULL,
		project			char(8)			NULL,
		category		char(8)			NULL,
		subcat			char(8)			NULL,
		target			char(1)			NOT NULL,
		targetid		int				NOT NULL,
		subscriber		int				NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	subtype
	(
		id				char(1)			NOT NULL,
		name			char(32)		NOT NULL,
		desctext		char(255)		NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	forward
	(
		id				int				NOT NULL,
		source			char(1)			NOT NULL,
		srcid			int				NOT NULL,
		srcsub			int				NOT NULL,
		desturl			varchar(255)	NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	email
	(
		id				int				NOT NULL,
		recvdate		datetime		NOT NULL,
		hdrmsgid		varchar(255)	NOT NULL,
		hdrfrom			varchar(255)	NOT NULL,
		hdrto			varchar(255)	NOT NULL,
		hdrsubject		varchar(255)	NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	emailcontent
	(
		id				int				NOT NULL,
		size			int				NOT NULL,
		content			longblob		NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	severity
	(
		id				char(1)			NOT NULL,
		name			char(32)		NOT NULL,
		info			char(255)		NULL,
		lvl				int				NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	state
	(
		id				char(1)			NOT NULL,
		name			char(32)		NOT NULL,
		info			char(255)		NULL,
		lvl				int				NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	priority
	(
		id				char(1)			NOT NULL,
		name			char(32)		NOT NULL,
		info			char(255)		NULL,
		lvl				int				NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	confidence
	(
		id				char(1)			NOT NULL,
		name			char(32)		NOT NULL,
		info			char(255)		NULL,
		lvl				int				NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	class
	(
		id				char(1)			NOT NULL,
		name			char(32)		NOT NULL,
		info			char(255)		NULL,
		lvl				int				NOT NULL,

	PRIMARY KEY( id )
	);

CREATE TABLE
	category
	(
		pkg				char(8)			NOT NULL,
		cat				char(8)			NOT NULL,
		sub				char(8)			NOT NULL,
		name			char(32)		NOT NULL,
		resp			int				NOT NULL,
		catdesc			char(255)		NULL,

	PRIMARY KEY( pkg, cat, sub )
	);

CREATE TABLE
	envdefs
	(
		id				char(8)			NOT NULL,
		os				char(32)		NOT NULL,
		osrel			char(32)		NOT NULL,
		platform		char(32)		NOT NULL,

	PRIMARY KEY ( id )
	);

CREATE TABLE
	repbug
	(
		report			int				NOT NULL,
		bug				int				NOT NULL,

	PRIMARY KEY ( report, bug )
	);

CREATE TABLE
	properties
	(
		propkey			char(128)		NOT NULL,
		propval			mediumblob		NULL,

	PRIMARY KEY ( propkey )
	);

CREATE TABLE
	authuser
	(
		userid			char(16)		BINARY NOT NULL,
		passwd			char(16)		BINARY NULL,

	PRIMARY KEY ( userid )
	);

CREATE TABLE
	authgroup
	(
		userid			char(16)		BINARY NOT NULL,
		groupid			char(16)		NOT NULL,

	PRIMARY KEY ( userid, groupid )
	);

