
package org.gjt.mail;

import java.io.*;
import java.util.Date;
import java.util.Vector;

import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*;


public class
EnhancedMimeMsg extends MimeMessage
	{
	private String		orgStr = null;
	private String		mailerStr = null;

	public
	EnhancedMimeMsg( Session session )
		{
		super( session );
		}

	public void
	buildMimeMessage(
			Address from, Address replyTo,
			Address[] to, Address[] cc, Address[] bcc,
			String subject, String text, Vector attach )
		throws MessagingException
		{
		int		textCounter = 0;

		if ( attach != null && attach.size() > 0 )
			{
			MimeMultipart multiPart = new MimeMultipart();

			MimeBodyPart msgPart = new MimeBodyPart();
			msgPart.setContent( text, "text/plain" );
			multiPart.addBodyPart( msgPart );

			for ( int aIdx = 0 ; aIdx < attach.size() ; ++aIdx )
				{
				MimeBodyPart attachPart = null;
				Object item = attach.elementAt(aIdx);

				if ( item instanceof File )
					{
					attachPart = new MimeBodyPart();

					File aFile = (File) item;
					FileDataSource fds =
						new FileDataSource( aFile );

					DataHandler dh = new DataHandler( fds );

					attachPart.setFileName( aFile.getName() );
					attachPart.setDisposition( Part.ATTACHMENT );
					attachPart.setDescription
						( "Attached file '" + aFile.getName() + "'" );

					attachPart.setDataHandler( dh );
					}
				else if ( item instanceof String )
					{
					++textCounter;
					String name = "Text_" + textCounter;

					attachPart = new MimeBodyPart();

					String aStr = (String) item;
					StringDataSource sds =
						this.new StringDataSource
							( aStr, name, "text/plain" );

					DataHandler dh = new DataHandler( sds );

					attachPart.setFileName( name );
					attachPart.setDisposition( Part.ATTACHMENT );
					
					int idx = 8;
					for ( ; ; )
						{
						int i = aStr.indexOf( " ", idx );
						if ( i < 0 || i > 24 )
							break;
						idx = i + 1;
						}

					attachPart.setDescription
						( "Text '" + aStr.substring( 0, idx )
							+ "...'" );

					attachPart.setDataHandler( dh );
					}
				else if ( item instanceof MimePart )
					{
					Part aPart = (MimePart) item;

					try {
						PartDataSource aSrc =
							this.new PartDataSource( aPart );

						DataHandler dh = new DataHandler( aSrc );

						String name = aPart.getFileName();
						if ( name == null || name.length() < 1 )
							name = "Untitled";

						attachPart = new MimeBodyPart();

						attachPart.setFileName( name );
						attachPart.setDisposition( Part.ATTACHMENT );
						attachPart.setDescription
							( "Forwarded attachment '" + name + "'" );

						attachPart.setDataHandler( dh );
						}
					catch ( Exception ex )
						{
						throw new MessagingException
							( "attaching message part: "
								+ ex.getMessage() );
						}
					}
				else if ( item instanceof DataSource )
					{
					try {
						DataHandler dh = new DataHandler( (DataSource)item );

						String name = dh.getName();
						if ( name == null || name.length() < 1 )
							name = "Untitled";

						attachPart = new MimeBodyPart();

						attachPart.setFileName( name );
						attachPart.setDisposition( Part.ATTACHMENT );
						attachPart.setDescription
							( "DataSource attachment '" + name + "'" );

						attachPart.setDataHandler( dh );
						}
					catch ( Exception ex )
						{
						throw new MessagingException
							( "attaching message part: "
								+ ex.getMessage() );
						}
					}
				else
					{
					throw new MessagingException
						( "unknown attachment object, class="
							+ item.getClass() );
					}

				if ( attachPart != null )
					{
					multiPart.addBodyPart( attachPart );
					}
				}

			this.setContent( multiPart );
			}
		else
			{
			this.setContent( text, "text/plain" );
			}

		this.setFrom( from );

		if ( replyTo != null )
			{
			Address[] addrs = new Address[1];
			addrs[0] = replyTo;
			this.setReplyTo( addrs );
			}

		this.setRecipients
			( Message.RecipientType.TO, to );

		if ( cc != null && cc.length > 0 )
			{
			this.setRecipients
				( Message.RecipientType.CC, cc );
			}

		if ( bcc != null && bcc.length > 0 )
			{
			this.setRecipients
				( Message.RecipientType.BCC, bcc );
			}

		this.setSubject( subject );

		this.setSentDate( new Date() );

		this.setHeader
			( "X-Mailer",
				( this.mailerStr != null )
					? this.mailerStr
					: ( "org.gjt.mail.EnhancedMimeMsg, "
						+ "<http://www.gjt.org/>" ) );

		this.setHeader
			( "Organization",
				( this.orgStr != null )
					? this.orgStr
					: ( "The Giant Java Tree, "
						+ "<http://www.gjt.org/>" ) );
		}

	public void
	setMailer( String mailerStr )
		{
		this.mailerStr = mailerStr;
		}

	public void
	setOrganization( String orgStr )
		{
		this.orgStr = orgStr;
		}

	public
	class		StringDataSource
	extends		Object
	implements	DataSource
		{
		private String			data;
		private String			name;
		private String			contentType;

		public
		StringDataSource( String data, String name, String contentType )
			{
			this.data = data;
			this.name = (name == null) ? "Untitled" : name;
			this.contentType =
				(contentType == null) ? "text/plain" : contentType;
			}

		public String
		getContentType()
			{
			return this.contentType;
			}

		public String
		getName()
			{
			return this.name;
			}

		public InputStream
		getInputStream()
			throws IOException
			{
			return new ByteArrayInputStream( this.data.getBytes() );
			}

		public OutputStream
		getOutputStream()
			throws IOException
			{
			throw new IOException
				( "StringDataSource is read-only" );
			}
		}

	public
	class		PartDataSource
	extends		Object
	implements	DataSource
		{
		private Part			part;
		private String			name;
		private String			contentType;
		private MimeType		mimeType;


		public
		PartDataSource( Part part )
			throws MimeTypeParseException, MessagingException
			{
			this.part = part;
			this.mimeType = new MimeType( part.getContentType() );
			this.contentType = this.mimeType.getBaseType();
			this.name = this.part.getFileName();
			if ( this.name == null )
				this.name = "Untitled";
			}

		public String
		getContentType()
			{
			return this.contentType;
			}

		public String
		getName()
			{
			return this.name;
			}

		public InputStream
		getInputStream()
			throws IOException
			{
			InputStream result = null;
			
			try {
				result = this.part.getInputStream();
				}
			catch ( MessagingException ex )
				{
				throw new IOException
					( "MessagingException: " + ex.getMessage() );
				}

			return result;
			}

		public OutputStream
		getOutputStream()
			throws IOException
			{
			throw new IOException
				( "PartDataSource is read-only" );
			}
		}

	}
